/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/*
 * Write and retrieve checkpoints information for lm_auto
 * $Id: auto_chk.c,v 1.7 2006-03-28 19:12:27 myrna Exp $
 */

static char *orescName[2];              /* Name of the output rescue
                                           files */
static void auto_dump (double ****nsc, double **pen, double **seg,
                       double **prob, int **counts, int ***recounts,
                       int total, int iter)
{
   FILE *fdump;

   unsigned int cseed;
   unsigned int tseed;
   int   c, i, j, k, m;

   static int whichdump = -1;           /* This variable alternates
                                         * between 0 and 1, indicating
                                         * which dump file to use */
   /* skip if no rescue file is specified */
   if (get_filent (ORESC)->fname != NULL)
   {
      /* First call sets up the filenames */
      if (whichdump == -1)
      {
         for (i = 0; i < 2; ++i)
         {
            orescName[i] = malloc (strlen (get_filent (ORESC)->fname) + 5);
            strcpy (orescName[i], get_filent (ORESC)->fname);
            strncat (orescName[i], i == 0 ? "1" : "2", 1);
         }
         whichdump = 0;
      }

      /* open rescue file, ready to be written */
      fdump = fopen (orescName[whichdump], "w");
      if (fdump == NULL)
      {
         fprintf (stderr, "Failed to open rescue file %s.\n",
                  orescName[whichdump]);
         exit (EXIT_FAILURE);
      }

      /* random seeds */
      getsd (&cseed, &tseed);
      fprintf (fdump, " %08x %08x\n", cseed, tseed);
      fflush (fdump);

      /* xx0, xx1 */
      for (j = 0; j <= pedsize; ++j)
      {
         for (i = 0; i < NLoci; ++i)
            fprintf (fdump, "%d %d\n", xx0[i][j], xx1[i][j]);
         fflush (fdump);
      }

      /* nscore[][][][] */
      for (c = 0; c < NScCmp; ++c)
         for (j = 0; j < NScSet[c]; ++j)
            for (k = 0; k < NLoci; ++k)
            {
               for (m = 0; m < PTgg[NPrGam[c][j]] + 1; ++m)
                  fprintf (fdump, "%f\n", nsc[c][j][k][m]);
               fflush (fdump);
            }

      /* prob[][] */
      for (c = 0; c < n_compnts; ++c)
      {
         for (j = 0; j < NLoci; ++j)
            fprintf (fdump, "%f ", prob[c][j]);
         fputc ('\n', fdump);
         fflush (fdump);
      }

      /* counts */
      for (c = 0; c < n_compnts; ++c)
      {
         for (j = 0; j < NLoci; ++j)
            fprintf (fdump, "%d ", counts[c][j]);
         fputc ('\n', fdump);
         fflush (fdump);
      }

      /* recounts */
      for (c = 0; c < n_compnts; ++c)
         for (j = 0; j < NLoci; ++j)
         {
            for (k = 0; k < 6; ++k)
               fprintf (fdump, "%d ", recounts[c][j][k]);
            fputc ('\n', fdump);
            fflush (fdump);
         }

      for (c = 0; c < n_compnts; ++c)
      {
         for (j = 0; j < NLoci - 1; ++j)
         {
            fprintf (fdump, "%f %f\n", pen[c][j], seg[c][j]);
         }
         fprintf (fdump, "%f\n", pen[c][NLoci - 1]);
      }

      /* total_counts and no. of iterations finished */
      fprintf (fdump, "%d\n", total);
      fflush (fdump);

      /* finished dummping */
      fprintf (fdump, "%d CHECKPOINTS_OK\n", iter);
      fclose (fdump);
      whichdump = 1 - whichdump;
   }
}

static void auto_restart (double ****nsc, double **pen, double **seg,
                          double **prob, int **counts, int ***recounts,
                          int *ptotal, int *piter)
{
   /* check point */
   FILE *frcvr = NULL;                  /* input rescue file handle */
   unsigned int cseed;
   unsigned int tseed;
   int   c, i, j, k, m;

   /* Try to access the rescue files */
   if (get_filent (IRESC)->fname != NULL)
   {
      frcvr = open_fil (IRESC);
      if (frcvr != NULL)
      {
         fprintf (stderr, "Opened input rescue file %s.\n",
                  get_filent (IRESC)->fname);

         /* Process input rescue file ... */
         /* recover seeds */
         fscanf (frcvr, "%x %x", &cseed, &tseed);
         setsd (cseed, tseed);

         /* xx0, xx1 */
         for (j = 0; j <= pedsize; ++j)
            for (i = 0; i < NLoci; ++i)
               fscanf (frcvr, "%d %d", &xx0[i][j], &xx1[i][j]);
         /* gg0, gg1 */
         for (i = 0; i < NLoci; ++i)
            set_FGL (i);

         /* nscore[][][][] */
         for (c = 0; c < NScCmp; ++c)
            for (j = 0; j < NScSet[c]; ++j)
               for (k = 0; k < NLoci; ++k)
                  for (m = 0; m < PTgg[NPrGam[c][j]] + 1; ++m)
                     fscanf (frcvr, "%lf ", &nsc[c][j][k][m]);

         /* prob */
         for (c = 0; c < n_compnts; ++c)
            for (j = 0; j < NLoci; ++j)
               fscanf (frcvr, "%lf ", &prob[c][j]);

         /* counts */
         for (c = 0; c < n_compnts; ++c)
            for (j = 0; j < NLoci; ++j)
               fscanf (frcvr, "%d ", &counts[c][j]);

         /* recounts */
         for (c = 0; c < n_compnts; ++c)
            for (j = 0; j < NLoci; ++j)
               for (k = 0; k < 6; ++k)
                  fscanf (frcvr, "%d ", &recounts[c][j][k]);

         for (c = 0; c < n_compnts; ++c)
         {
            for (j = 0; j < NLoci - 1; ++j)
               fscanf (frcvr, "%lf%lf", &pen[c][j], &seg[c][j]);
            fscanf (frcvr, "%lf", &pen[c][NLoci - 1]);
         }

         /* total_counts and no. of iterations finished */
         fscanf (frcvr, "%d %d", ptotal, piter);
         ++*piter;
         printf ("Restart from %d iteration.\n", *piter);

         fclose (frcvr);
      }
      else
      {
         fprintf (stderr, "Failed to open input rescue file %s.\n",
                  get_filent (IRESC)->fname);
         give_up (ERR_qit);
      }
   }
}

static void finish_dumping ()
{
   if (orescName[0])
      free (orescName[0]);
   if (orescName[1])
      free (orescName[1]);
}
