/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */


#ifndef MULMSAMPLE_H
#define MULMSAMPLE_H

#include "incs.h"
#include "nghds.h"
#include "markers.h"
#include "rans.h"
#include "msample.h"

/* global mcmc is performed for all the components if GlobalMcmc == 1; 
   in this case, peel by components is still available, 
   but exact calculations are not allowed.
   if GlobalMcmc == 0, lod scores are estimated or calculated component-wise */
int GlobalMcmc;

/* MUL is an indicator in program lm_multiple
   MUL = 0  : do the regular lm_markers: now a local variable in main prog.
   otherwise: do lm_mul_new
   MAXMNupdate is maximum number of meioses allowed for joint updating 
*/

int Max_update;
double P_meio, P_ind, P_sib, P_3gen;

/* These variables replace previously defined constants  
   MAXNMupdate, PMEIO, PIND, PSIB, P3GEN  ;      EAT April 2007      */

/* structures defined by Liping */

typedef struct _indices {
        int index;
        struct _indices *next;
} Indices;

typedef struct _Marrg
{
   int index;     /* index of marriage, from 0 */
   int c;         /* index of component, from 1 */
   int wife;      /* index of wife */
   int husb;      /* index of husband */
   int nkid;      /* number of kids */
   int *kid;      /* indices of kids */
   int nkidmeio;  /* number of meioses to kids */
   int *kidmeio;  /* list of meioses to kids */
} Marrg;

typedef struct _Marrglist
{
   struct _Marrg *current;
   struct _Marrglist *next;
} Marrglist;

typedef struct _IndMarrg
{
   int index;    /* index of individual */
   int c;        /* index of component */
   int nmarr;    /* number of marriages */
   int *marr;    /* indices of marriages, from 0 */
} IndMarrg;

typedef struct _CmpMarrg
{
   int c;        /* index of components, start from 1 */ 
   int nmarr;    /* no. of marriages in component c */
   int *marr;    /* list of indices of marriages, from 0 */
}  CmpMarrg;

/* global variables used in lm_multiple, might go to globs.h later.
   marr is a new global variable with new structure;
   indmarr can be part of nghd;
   cmpmarr can be part of cmplist.
*/
int        NMarrs;      /* total number of marriages */
Marrg      *marr;       /* [NMarrs] */
IndMarrg   *indmarr;    /* [pedsize] */
CmpMarrg   *cmpmarr;    /* [n_compnts] */

/* Functions defined by Liping, Jul 10, 2005 */

/* dir: MulMSample, file: misc.c */
double   logsum (double, double);
double   probtrans (int *, int *, int, double *, int *);

/* dir: MulMSample, file: setupMapNew.c */
void setMapsNew (void);

/* dir: MulMSample, file: fwd_back_smpl2.c */
void   fwd_compute3 (int, int, int, double *, int *, int *, double *, 
                     double *, double *);
int    back_sample3 (int, int, int, int, double *, int *, int *, double *);
void   fwd_compute4 (int, int, int, int, double *, int *, int *, double *, 
                     double *, double *);
int    back_sample4 (int, int, int, int, int, double *, int *, int *, 
                     double *);
void   forward2  (int, int, double **, double **, double *, int *);
void   backward2 (int, int, double **, double **, double *, int *);

/* dir: MulMSample, file: do_msmpl2.c   LPT  */
/*   alloc & free routines, and subroutines of do_msmpl2:  added EAT 2006 */
void alloc_msmpl2(int, int);
void free_msmpl2 (int);
void do_msmpl2    (int, int, int, int *, int *, int, double **, int);
void do_msmpl_hap (int, int, int, int *, int, int *, int *, int, 
                   double **, int);

/* MulMSample, file: setupMarriage.c */
void setup_marriage (void);
void prnt_marriage (void);
void free_marriage (void);

/* dir: MulMSample, file: sampling2.c */
void    setup_sampling2 (int);
void    free_sampling2 (void);
void    sampling2 (int, int, int, int, double **);

/* dir: MulMSample, file: mul_msmpl.c */
void update_meio (int, int *, int, int *, int, int *, int, 
                  int, int, double **, int);
void update_ind  (int, int *, int, int *, int, int *, int, 
                  int, int, double **, int);
void update_sib  (int, int *, int, int *, int, int *, int, 
                  int, int, double **, int);
void update_3gen (int, int *, int, int *, int, int *, int, 
                  int, int, double **, int);
void change_meiosis (int, int, int, int *, int *, int);

/* dir: Sample, file: sampling_cmp.c */
/* LPT, Jun 2007 */
void    setup_sampling_cmp (int, int, int);
void    free_sampling_cmp (void);
void    sampling_cmp (int, int, int, int, int, int, double **);

double do_lsmpl_cmp (int, int, int, double **, int);
void do_rblik_cmp (int, int, int, int, int *, int *, double *, double *,
                   double *, double *, double **, double ***, double *, 
                   double *);
double do_mjrlik_cmp (int, int, int, double, double, double, double, 
                      int, double *);

/* dir: Sample, file: mcmc_lods.c */
/* LPT, Jun 2007 */
void mcmc_lods(int, int, int, int, int, double **, double **, int *, int *, 
               double **, double *, double *);

/* end of Liping's programs */

#endif
