/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*   nghds.h:  neighborhood structures used by polyEM programs         */
/*             also macros and function prototypes for Nghds/ fncs     */
/*             including data interfacing between Pars/ proc_poly()    */
/*             and Nghds/ fncs                                         */
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef NGHDS_H
#define NGHDS_H

/**
 * @defgroup nghds libnghds
 * @ingroup Libraries
 * @{
 */

#include "incs.h"
#include "pedchk.h"

/** Quantitative trait data, for each trait and each pedigree member */
GLOBAL double ** quant;


typedef struct _Ind {
        struct _Ind *pa,*ma;
        char self[MAXNAMLEN+1];               /* char name */
        int index;
        int component;
        int sex;
        struct _Marlist *marriages;
        int patmeio, matmeio;          /* indices of pat/mat meioses    */
        int nkidmeio;                  /* number of kid meioses         */
        int *kidmeio;                  /* indices of kid meioses        */
        int obs;                       /* trait obs indicator           */
        int *d_dat;                    /* pointer to discrete trait data*/
        double *q_dat;                 /* pointer to quant traits data */
        int *g_dat;                 /* pointer to 2x(nloc+1) genotypic data */
} Ind;

/*      int ntrips;        removed from _Ind  */
/*      int *trips;        used by PBGibbs    */
/*      int nbips;         (MORGAN V1.1)      */
/*      int *bips;                            */

typedef struct _Component {
        int index;
        int size;              /* number of individuals : EAT 10/2004 */
        int nfou;              /* number of founders    : EAT 10/2004 */
        struct _Indlist *people;
        int nmeioses;          /* meioses to be sampled/summed  */
        int *meioses;          /*     EAT 10/2004               */
        int naff;              /* slot for  number affected EAT 10/2004 */
        int nunaff;            /* slot for number unaffected */
        int *ind_aff;          /* vector of affecteds indices */
        int *ind_unaff;        /* vector of unaffected indices */
} Component;

typedef struct _Mar {
        struct _Ind *spouse;
        struct _Indlist *kids;
} Mar;

typedef struct _Marlist {
        struct _Mar *current;
        struct _Marlist *next;
} Marlist;

typedef struct _Indlist {
        struct _Ind *current;
        struct _Indlist *next;
} Indlist;

typedef struct _Switch
{
   Ind   ind;
   int   sex;
   int   cmpnt;
}
Switch;

/*  add component index to Switch; replace Switchlist and Switchcmp
    structures by vector of switch indices in Component structure
    Moved from Markers to Nghds
     EAT 10/2004
 */

/** The vector of the meioses of the pedigree. */
GLOBAL Switch *meiosis;
                                        /* parameters set by Nghds/     */
                                        /*   functions:                 */

                                        /* FIXME pedsize, namestring, n_cmp */ 
GLOBAL int pedsize;                     /* no. of individuals read      */
GLOBAL int namestring;                  /* largest name in pedigree file*/
GLOBAL int n_compnts;                   /* # of components found        */
GLOBAL int max_cpsize;                  /* maximum component size found */

GLOBAL int **data;                      /* was data[MAX_SIZE][7]        */

GLOBAL Ind *nghd;
GLOBAL Component *cmplist;              /* vector of components         */
GLOBAL Ind nullnode;

GLOBAL int *qdex, *idex;                /* pedigree to data indexing;
                                           moved to here, since used by both
                                           PolyEm/Quant and Autozyg/Sample
                                           EAT Jan 2004                  */
                /* will be allocated size (pedsize +1) in input routines */

void setPed(double **);
int  setSwitches(void);
void  dbgGDat (void);

void alloc_ped(void);
void free_ped(void);
void free_nghds(void);
void free_cmplists(void);
void nghds(void);
void read_name (FILE *, char *);        /* read in a name               */
void input_ped(FILE *, double **);
int neighbour(int, int);
int infect(int);
int components(void);
int make_switches (void);              /* moved from markers.h, EAT 10/'04 */
                                       /* and got rid of arguments 11/2004 */
                                       /* renamed from list_switches 3/2005 */         
void  prnt_switches (int);
void  free_switches (void);

int get_posit(const char *);
int isNullNode (const Ind *);           /* test if an Ind pointer points to
                                          the nullnode (UNKNOWN individual) */
void getIndIxs(int, int, char **, int *); 
int count_founders(void);
int intmax(int *aa, int n1, int n2);

double kinship (Ind *, Ind *);
double inbreeding (Ind *);

/* Exact IBD status calculation on two linked loci through recursive
   formula. */
double g4link_idx (int, int, int, int, int, int, int, int,
                   double);
double g4link (Ind *, int, Ind *, int, Ind *, int, Ind *, int, double);

/** @} */

#endif  /*NGHDS_H*/

