/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#include "traits.h"

/* includes markers.h and others included therein
 * see traits.h for specs and purpose of routines in this file.
 *                                                            EAT March 2002
 */

int print_traits (int nt, int *loc_fl)
{
   int   i, n, nm = 0;

   n = 4;
   if (LocNamLen > n)
      n = LocNamLen;

   printf ("\n Genetic variables in the analysis:\n");

   printf (" %8s    %-*s   %-12s\n\n", "index", n, "name", "type");

   for (i = 0; i < nt; ++i)
   {
      if (traits[i].trait_type != UNDEF)
         printf (" %8d    %-*s", i, n, traits[i].self);

      switch (traits[i].trait_type)
      {
          case MARKER_GENO:
             printf ("   %-12s", "geno-marker");
             traits[i].locus = loc_fl[traits[i].locus];
             ++nm;
             break;
          case GENOTYPIC:
             printf ("   %-12s", "genotypic");
             break;
          case DISCRETE:
             printf ("   %-12s", "discrete");
             break;
          case ORDINAL:
             printf ("   %-12s", "ordinal");
             break;
          case QUANTITATIVE:
             printf ("   %-12s", "quantitative");
             break;
          case INHERITANCE:
             printf ("   %-12s", "inheritance");
             break;
          default:
             break;
      }
      if (traits[i].trait_type != UNDEF)
         printf ("\n");
   }
   printf ("\n");
   fflush (stdout);

   return (nm);
}

void recess2 (double *vv, const struct Trait *trait,
              const Ind * id, const Loclist * ll, double lam)
/* will become a penetrance routine for a discrete trait */
{
   return;
}

/* ****************** JUNK STUFF ABOUT LINKED LIST *******************
   Indlist *child_ptr;

   for (j = i + 1; j < limits[1] + 1 && parent[j][2] == parent[i][2]; ++j)
   {
   child_ptr->next = (Indlist *) malloc (sizeof (Indlist));
   alloc_chk (WHERE, child_ptr->next);
   child_ptr = child_ptr->next;
   child_ptr->current = &nghd[parent[j][0]];
   }
   }
   child_ptr->next = NULL;
   ***************************************************** */

void default_traits (int nt)
/* set up default names and specs for traits */
{
   int   i, m = 0;
   struct Trait *trait;

   for (i = 0; i < nt; i++)
   {
      trait = traits + i;
      trait->index = i;
      trait->locus = i;
      if (i == 0)                       /* trait locus */
      {
         if (TraitTyp == UNDEF)         /* null locus */
         {
            trait->trait_type = UNDEF;
            strncpy (trait->self, "", MAXNAMLEN + 1);
         }
         else if (TraitTyp == GENOTYPIC)
         {
            trait->trait_type = GENOTYPIC;
            strncpy (trait->self, "trt-geno", MAXNAMLEN + 1);
         }
         else if (TraitTyp == DISCRETE) /* phenotypic data */
         {
            trait->trait_type = DISCRETE;
            trait->penetrance = recess2;
            strncpy (trait->self, "trt-pheno", MAXNAMLEN + 1);
         }
         else if (TraitTyp == QUANTITATIVE) /* continuous data */
         {
            trait->trait_type = QUANTITATIVE;
            strncpy (trait->self, "trt-quant", MAXNAMLEN + 1);
         }
         else if (TraitTyp == INHERITANCE) /* seg indicators */
         {
            trait->trait_type = INHERITANCE;
            strncpy (trait->self, "", MAXNAMLEN + 1);
         }
      }
      else
      {                                 /* marker locus */
         trait->trait_type = MARKER_GENO;
         strncpy (trait->self, MarkNam[m], MAXNAMLEN + 1);
         ++m;
      }
      if (LocNamLen < strlen (trait->self))
         LocNamLen = strlen (trait->self);
   }
   return;
}

void flag_loci (int nt, int nloc, int *loc_fl)
/*  sets a trait flag for each marker locus, by input record */
/*  sets transform of loci for setting marker data           */
{
   int   i, j;

   for (i = 0; i <= nloc; ++i)
      loc_fl[i] = -1;

   for (i = 0; i < nt; ++i)
      if (traits[i].trait_type == MARKER_GENO)
         loc_fl[traits[i].locus] = i;

   j = 0;
   for (i = 0; i <= nloc; ++i)
      if (loc_fl[i] > -1)
      {
         ++j;
         loc_fl[i] = j;
      }

   return;
}

int set_traits (int nl, int *loc_fl)
{
   int   nt;                            /* number of traits,
                                         * currently = number of loci */
   nt = nl + 1;

/* Allocate trait structures */

   traits = malloc (sizeof (struct Trait) * nt);
   alloc_chk (WHERE, traits);

/* set trait ; default case  */
   default_traits (nt);

/* flag marker loci that are defined by the traits  */
   flag_loci (nt, nl, loc_fl);

/* return the number of traits */
   return (nt);
}
void free_traits (void)
{
   if (traits)
   {
      free (traits);
      traits = NULL;
   }
   return;
}

int set_eval_points (int nloc, int *loc_left, int *loc_rght,
                     double *rl0, double *rr0, double *rl1, double *rr1)
{
#if 0
   ****from globs.h **** struct locus_pos
   {
      int   locus_typ;                  /* is this: selected marker                     */
      /*          non-selectd marker test pos         */
      /*          trait test position                 */
      int   intv_no;                    /* interval no., 0 ... n                        */
      /*     0 : before marker 1,                     */
      /*     i : marker i,                            */
      /*         between marker i, i+1, if test pos   */
      /*     n : marker n,                            */
      /*         or following marker n, if test pos   */
      int   no;                         /* marker or trait no. (no. from maps)          */
      double rf[2];                     /* sex-specific recomb fract with preced marker */
      /* or marker 1, for interval "0"                */
      double rfnxt[2];                  /* sex-specific recomb fract with next marker   */
      double pos[2];                    /* sex-specific distance to marker 1, in cM     */
   };
#endif

   int   p, sx;
   double *rl[2], *rr[2];
   LOCPOS_PTR p_p;

#if 0
   printf ("\n Table of evaluation points ");
   fflush (stdout);
   printf ("\n    loc_left  male-rec fem-rec ");
   printf ("      loc_right  male-rec fem-rec ");
#endif

   rl[0] = rl0;
   rl[1] = rl1;
   rr[0] = rr0;
   rr[1] = rr1;

   loc_left[0] = 0;
   loc_rght[0] = 0;
   for (sx = FEM; sx <= MAL; ++sx)
   {
      rl[sx][0] = 0.;
      rr[sx][0] = 0.;
   }

   for (p = 1, p_p = LocPos; p <= NLocPos; ++p, ++p_p)
   {
      if (p_p->locus_typ == MARK_LOC)   /* tests for marker position        */
      {
         loc_left[p] = p_p->intv_no;
         loc_rght[p] = p_p->intv_no;
         for (sx = FEM; sx <= MAL; ++sx)
         {
            rl[sx][p] = 0.;
            rr[sx][p] = 0.;
         }
      }
      else
      {
         if (p_p->intv_no == 0)         /* tests for pos before 1st marker    */
         {
            loc_left[p] = -1;
            loc_rght[p] = 1;
         }
         else if (p_p->intv_no < NMarks) /* tests for test pos in interval    */
         {
            loc_left[p] = p_p->intv_no;
            loc_rght[p] = p_p->intv_no + 1;
         }
         else                           /* test pos after last marker         */
         {
            loc_left[p] = p_p->intv_no;
            loc_rght[p] = -1;
         }
         for (sx = FEM; sx <= MAL; ++sx) /* sets recomb fracts                */
         {
            rl[sx][p] = p_p->rf[sx];
            rr[sx][p] = p_p->rfnxt[sx];
         }
      }
   }

#if 0
   printf ("\n   %d  %f  %f    %d  %f  %f",
           loc_left[p], rl[1][p], rl[0][p], loc_rght[p], rr[1][p], rr[0][p]);
   fflush (stdout);
#endif

   return (p);
}
