/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/* these functions are called from proc_lods().
 */

/**
 * checks select marker and trait statements for errors.
 */
int test_lods_sel (void)
{
   if (!first_sim_chrom_node)           /*select ... markers .. required      */
   {
      err_typ = ERA_SEL;
      return FALSE;
   }

   else if (first_sim_chrom_node->p_next) /*checks for only one chromosome    */
   {
      err_typ = ERA_OOS;
      return FALSE;
   }

   if (!sim_traits)                     /* checks for select trait            */
   {
      err_typ = ERA_MST;
      return FALSE;
   }

   return TRUE;
}

/**
 * checks scalar parameters and sets defaults
 */
int test_lods_pars (void)
{
   fprintf (stdout, " \n");

   if (NIt == UNDEF)
   {
      if (!LM_MAP)
      {
         err_typ = ERA_NIT;
         return FALSE;
      }
      else
         NIt = MCEMIT;
   }

   if (SmplSetup == UNDEF)
      SmplSetup = SEQIMP;

   if (NSeqImp == UNDEF)
   {
      NSeqImp = RSEQIMP * NIt;
      if (NSeqImp < 2)
         NSeqImp = 2;
   }

   if (Scan == UNDEF)
      Scan = Y;
   if (ScorIt == UNDEF)
   {
      if (!LM_MAP && !LM_IBDTESTS)
         ScorIt = 1;
      else if (LM_IBDTESTS)
         ScorIt = NIt / 50 > 10 ? 50 : NIt / 10;
   }

   if (SeqImpIt == UNDEF)
      SeqImpIt = 0;

   if (GlobalMcmc == UNDEF)
      GlobalMcmc = 0;

   if (NBrnIt == UNDEF)
      if (LM_MAP)
         NBrnIt = RBRNIT_MAP * NIt;
      else if (LM_PVAL)
         NBrnIt = RBRNIT_PVAL * NIt;
      else if (LM_IBDTESTS)
         NBrnIt = RBRNIT_IBDTESTS * NIt;
      else if (LM_AUTO)
         NBrnIt = RBRNIT_AUTO * NIt;

   if (LM_MAP)
   {
      if (MapEstMod == UNDEF)           /* checks for with or wthout mistypng */
         err_typ = ERA_MMT;
      else if (MapEstMod != MISTYP)     /* no mistyping, checks algorithm     */
      {
         if (MaximAlgo == UNDEF)
            err_typ = ERA_MMA;
      }
      else if (MapEstMod == MISTYP)     /* mistyping, checks genotyping       */
      {                                 /* error rate                         */
         if (GenoErrRat == 0.)
            err_typ = ERA_NGE;
      }
      if (MapTyp == UNDEF)              /* if output map type no specified,   */
         MapTyp = BOTH;                 /* sets default                       */

      if (err_typ)
         return FALSE;
   }

   if (LM_PVAL)
   {
      if (SpairsOn == UNDEF && RobdomOn == UNDEF && MaxFGLOn == UNDEF)
      {
         MaxFGLOn = 1;                  /* assigns default values             */
         SpairsOn = 0;
         RobdomOn = 0;
      }
   }

   if (LM_IBDTESTS)
      if (IBDStats == UNDEF && LikRatStats == UNDEF)
      {
         err_typ = ERA_NCS;
         return FALSE;
      }
      else if (IBDStats == 1)           /* checks IBD statistics options      */
      {
         LikRatStats = 0;
         if (DeltaOn != UNDEF || Lambda_pOn != UNDEF) /* first measures       */
            err_typ = ERA_WIM;
         DeltaOn = 0;
         Lambda_pOn = 0;

         if (NormalEnable == UNDEF || PermutEnable == UNDEF)
         {
            NormalEnable = 1;           /* assigns tests defaults             */
            PermutEnable = 0;
         }

         if (SpairsOn == UNDEF || RobdomOn == UNDEF || SAffUnaffOn == UNDEF
             || LambdaOn == UNDEF || MaxFGLOn == UNDEF)
         {
            SpairsOn = 1;               /* assigns meas defaults              */
            RobdomOn = 0;
            SAffUnaffOn = 0;
            LambdaOn = 0;
            MaxFGLOn = 0;
         }

         if (NormalEnable && SAffUnaffOn)
            err_typ = ERA_NSU;

         else if (ChiSqEnable != UNDEF) /* checks tests                       */
            err_typ = ERA_WIT;
         ChiSqEnable = 0;
      }

      else if (LikRatStats == 1)        /* checks lhood-ratio stat options    */
      {
         IBDStats = 0;
         if (SpairsOn != UNDEF || RobdomOn != UNDEF /* first measures         */
             || SAffUnaffOn != UNDEF || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
            err_typ = ERA_WLM;
         SpairsOn = 0;
         RobdomOn = 0;
         SAffUnaffOn = 0;
         LambdaOn = 0;
         MaxFGLOn = 0;

         if (DeltaOn == UNDEF || Lambda_pOn == UNDEF)
         {
            DeltaOn = 1;                /* assigns meas defaults              */
            Lambda_pOn = 0;
         }
         if (NormalEnable != UNDEF || PermutEnable != UNDEF) /* checks tests  */
            err_typ = ERA_WLT;
         NormalEnable = 0;              /* assigns tests defaults             */
         PermutEnable = 0;

         if (ChiSqEnable = UNDEF)
            ChiSqEnable = 1;

         if (Lambda_pOn)                /* assigns grid spacing               */
         {
            IntervalSegL = .5 / (NumOfSegL - 1.);
            IntervalSegP = .8 / (NumOfSegP - 1.);
         }
      }

   if (ChkIt == UNDEF)
      ChkIt = NIt / 5;

   if (FilIt == UNDEF)
      FilIt = NIt;

   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks pseudo-priors
 */
int test_ps_pr (void)
{
   if (InPseudo)
      if (n_pseudo != NTPos + 1)
      {
         err_typ = ERA_NPP;
         sprintf (errnumb, "%d", NTPos + 1);
         err_txt = errnumb;
         return FALSE;
      }
      else if (PseudoPrior[0] == 0.0)
      {
         err_typ = ERA_ZUT;
         return FALSE;
      }
   return TRUE;
}

/**
 * checks the map for test positions for the tloc.
 * checks for correct no. of tloc. 
 * tests for tloc map present for this chromosome.
 * checks sex-specific maps for consistency.
 */
int test_tloc_vmap (SIM_PTR p_sel, int *ss_map)
{
   int   sx, j;
   VMAP_PTR p_vmap;
   VMAP_PTR p_m[3] = { NULL, NULL, NULL };

   if (p_sel->no_tloc_sel != 1)         /* check for one tloc only           */
   {
      err_typ = ERA_NAT;                /* FIXME - should go in test_atz_se?  */
      return FALSE;
   }
   for (sx = FEM; sx <= AVG; ++sx)      /* check for tloc map present        */
      if (p_vmap = first_tloc_vmap_node[sx])
      {
         while (p_vmap->p_next && p_vmap->chrom_no < p_sel->chrom_no)
            p_vmap = p_vmap->p_next;
         if (p_vmap->chrom_no == p_sel->chrom_no)
         {
            p_m[sx] = p_vmap;           /* if found, check for sex-spec maps  */
            if (sx != AVG)
               *ss_map = TRUE;
         }
      }
   if (!p_m[AVG] && !p_m[MAL] && !p_m[FEM]) /* check for at least 1 external  */
   {                                    /* or interval prop map present       */
      if (p_sel->no_mark_map > 1)       /* (need for tloc #)                 */
         err_typ = ERA_MXM;             /* FIXME  ??? remove later ???        */
      else                              /* (sel tloc statement)              */
         err_typ = ERA_MBM;             /* FIXME when adding tloc no.         */
      for_chrom (p_sel->chrom_no);
      return FALSE;
   }

   for (sx = FEM; sx <= AVG; ++sx)      /* checks for tloc no. the same      */
   {
      j = (sx + 1) % (AVG + 1);
      if (p_m[sx] && p_m[j])
         if (p_m[sx]->no != p_m[j]->no)
            err_typ = ERA_ITI;
   }
   if (err_typ)
   {
      for_chrom (p_sel->chrom_no);
      return FALSE;
   }
   return TRUE;
}

/**
 * checks the map for beginning and ending test positions for the tloc.
 */
int test_tloc_vext (SIM_PTR p_sel)
{
   int   sx, beg = 0, end = 0;
   VMAP_PTR p_vmap;
   VMAP_PTR p_m[3] = { NULL, NULL, NULL };

   for (sx = FEM; sx <= AVG; ++sx)      /* check for tloc map present         */
      if (p_vmap = first_tloc_vmap_node[sx])
      {
         while (p_vmap->p_next && p_vmap->chrom_no < p_sel->chrom_no)
            p_vmap = p_vmap->p_next;
         if (p_vmap->chrom_no == p_sel->chrom_no)
         {
            p_m[sx] = p_vmap;           /* if found, check for sex-spec maps  */
            if (p_m[sx]->p_beg)         /* for beginning or ending positions  */
               beg = TRUE;
            if (p_m[sx]->p_end)
               end = TRUE;
         }
      }
   if (!beg || !end)                    /* test for beginning and/or ending   */
      if (vmap_opt.default_xt)          /* positions not specified            */
         err_typ = ERA_DBE;
      else                              /* check for no defaults              */
         err_typ = ERA_NBE;

   if (err_typ)
   {
      for_chrom (p_sel->chrom_no);
      return FALSE;
   }
   return TRUE;
}

/**
 * checks the maps for interval proportions.
 * checks for interval no.'s within range (< no. markers).
 */
int test_tloc_vint (SIM_PTR p_sel)
{
   int   sx, intv = 0;
   VMAP_PTR p_vmap;
   VMAP_PTR p_m[3] = { NULL, NULL, NULL };
   IMAP_PTR p_n;

   for (sx = FEM; sx <= AVG; ++sx)      /* check for tloc map present         */
      if (p_vmap = first_tloc_vmap_node[sx])
      {
         while (p_vmap->p_next && p_vmap->chrom_no < p_sel->chrom_no)
            p_vmap = p_vmap->p_next;
         if (p_vmap->chrom_no == p_sel->chrom_no)
         {
            p_m[sx] = p_vmap;           /* if found, check for sex-spec maps  */
            if (p_m[sx]->p_intv)        /* for interval proportions           */
               intv = TRUE;
         }
      }
   if (p_sel->no_mark_map == 1)         /* if only 1 marker,                  */
   {
      if (intv)                         /* test for extraneous intervl props  */
         err_typ = ERA_XEI;
   }
   else if (p_sel->no_mark_map > 1)     /* if more than 1 marker,             */
      if (!intv)                        /* test for no interval proportios    */
         if (vmap_opt.default_nt)       /* specified                          */
            err_typ = ERA_DEI;
         else                           /* check for no defaults              */
            err_typ = ERA_NDI;

   if (intv && p_sel->no_mark_map > 1)  /* test for valid interval no.'s      */
      for (sx = MAL; sx <= AVG; ++sx)
         if (p_m[sx] && (p_n = p_m[sx]->p_intv))
         {
            while (p_n && p_n->p_next)
               p_n = p_n->p_next;
            if (p_n->intv_no > p_sel->no_mark_map - 1)
               err_typ = ERA_MIO;
         }
   if (err_typ)
   {
      for_chrom (p_sel->chrom_no);
      return FALSE;
   }
   return TRUE;
}

/**
 * makes vprntmap node.
 * note: Autozyg programs use only one chromosome 
 */
PRNTVMAP_PTR add_vprntmap (VMAP_PTR p_vm, int sx)
{
   PRNTVMAP_PTR p_p;

   p_p = malloc (sizeof (PRNTVMAP));
   alloc_chk (WHERE, p_p);

   first_vpmap[sx] = p_p;
   p_p->chrom_no = p_vm->chrom_no;
   p_p->no = p_vm->no;
   p_p->p_beg = NULL;
   p_p->p_end = NULL;
   p_p->p_intv = NULL;
   p_p->p_sintv = NULL;

   return p_p;
}

/**
 * makes a list of external trial tloc positions.
 * makes the vprntmap list for positions at the start or end of the chrom      
 */
void add_tloc_vext (PRNTVMAP_PTR p_pv, int sex)
{
   XMAP_PTR p_x;
   PXMAP_PTR p_px;

   int   i, k;

   for (k = BDM; k <= EDM; ++k)         /* sets beginning pos, then ending    */
   {
      p_px = malloc (sizeof (PXMAP));
      alloc_chk (WHERE, p_px);
      p_px->no_pos = 0;
      p_px->p_dist = NULL;
      p_px->p_rf = NULL;

      if (k == BDM)
         p_pv->p_beg = p_px;
      else
         p_pv->p_end = p_px;

      if (p_x = nxt_tloc_vext (sex, p_pv, k)) /* looks for beg or ending map  */
      {
         p_px->no_pos = p_x->no_pos;    /* uses map from selected gender      */
         p_px->p_dist = malloc (p_x->no_pos * sizeof (double));
         alloc_chk (WHERE, p_px->p_dist);
         p_px->p_rf = malloc (p_x->no_pos * sizeof (double));
         alloc_chk (WHERE, p_px->p_rf);
         for (i = 0; i < p_x->no_pos; ++i)
         {
            *(p_px->p_dist + i) = *(p_x->p_dist + i);
            *(p_px->p_rf + i) = *(p_x->p_rf + i);
         }
      }
      else if (!vmap_opt.default_xt)
         p_px->no_pos = 0;              /* does not use canned defaults       */
      else
      {
         p_px->no_pos = NXtrnPos;       /* uses canned defaults for map       */
         p_px->p_dist = malloc (NXtrnPos * sizeof (double));
         alloc_chk (WHERE, p_px->p_dist);
         p_px->p_rf = malloc (NXtrnPos * sizeof (double));
         alloc_chk (WHERE, p_px->p_rf);
         for (i = 0; i < NXtrnPos; ++i)
            if (k == BDM)
            {
               *(p_px->p_rf + i) = *(XtrnPos + NXtrnPos - i - 1);
               *(p_px->p_dist + i) = rf2dist (*(XtrnPos + NXtrnPos - i - 1));
            }
            else if (k == EDM)
            {
               *(p_px->p_rf + i) = *(XtrnPos + i);
               *(p_px->p_dist + i) = rf2dist (*(XtrnPos + i));
            }
      }
   }
   return;
}

/**
 * makes a list of trial tloc positions for ea marker interval.
 */
void add_tloc_vint (PRNTVMAP_PTR p_pv, PRNTMAP_PTR p_pm, int sex)
{
   IMAP_PTR p_n;
   PIMAP_PTR p_i, p_j;
   PRDIST_PTR p_l;
   int   i, m;

   p_l = p_pm->p_locus;

   for (m = 1; m < p_pm->no_mark_map; ++m)
   {
      p_j = malloc (sizeof (PIMAP));    /* allocs and initializes the node    */
      alloc_chk (WHERE, p_j);

      p_j->intv_no = m;
      p_j->selectd = p_l->selectd;
      p_j->no_prop = 0;
      p_j->p_prop = NULL;
      p_j->p_next = NULL;

      if (!(p_n = nxt_tloc_vint (sex, m, p_pv))) /* looks for intrvl props    */
         p_n = nxt_tloc_vint (sex, 0, p_pv); /* for this interval or "all"    */
      if (p_n)                          /* intervals, using default genders   */
      {
         p_j->no_prop = p_n->no_prop;
         p_j->p_prop = malloc (p_n->no_prop * sizeof (double));
         alloc_chk (WHERE, p_j->p_prop);
         for (i = 0; i < p_n->no_prop; ++i)
            *(p_j->p_prop + i) = *(p_n->p_prop + i);
      }
      else if (!vmap_opt.default_nt)
         p_j->no_prop = 0;              /* does not use canned defaults       */
      else
      {
         p_j->no_prop = NIntProp;       /* uses canned defaults for map       */
         p_j->p_prop = malloc (NIntProp * sizeof (double));
         alloc_chk (WHERE, p_j->p_prop);
         for (i = 0; i < NIntProp; ++i)
            *(p_j->p_prop + i) = *(IntProp + i);
      }
      p_l = p_l->p_next;

      if (m == 1)
         p_pv->p_intv = p_j;
      else
         p_i->p_next = p_j;
      p_i = p_j;
   }
   return;
}

/**
 * builds a list of tloc positions for ea selected marker interval,
 * and for the start and end of the chromosome.
 * all tloc positions and non-selected markers are preserved; the
 * map positions are determined relative to the selected markers
 */
void sel_tloc_vpos (PRNTVMAP_PTR p_pv, PRNTMAP_PTR p_pm)
{
   int   i, k, j;
   double s;
   PIMAP_PTR p_i, pi_sav;
   PSMAP_PTR p_new;
   PRDIST_PTR p_l;

   p_pv->p_sintv = alloc_sintv (0);     /* allocs a node for ea selected      */
   p_new = p_pv->p_sintv;               /* interval, and the beginning & end  */

   for (i = 1; i <= p_pm->no_mark_sel; ++i)
   {
      p_new->p_next = alloc_sintv (i);
      p_new = p_new->p_next;
   }
   p_new->p_next = NULL;

   p_i = p_pv->p_intv;                  /* BEGINNING:  counts no. positions   */
   p_new = p_pv->p_sintv;               /* and allocates list                 */
   p_new->no_pos = p_pv->p_beg->no_pos;
   while (p_i && !p_i->selectd)
   {
      p_new->no_pos += p_i->no_prop + 1;
      p_i = p_i->p_next;
   }
   p_i = p_pv->p_intv;
   p_l = p_pm->p_locus;

   alloc_sintl (p_new);

   s = p_l->sel_dist;                   /* sets positions before marker 1     */

   for (k = 0; k < p_pv->p_beg->no_pos; ++k)
   {
      *(p_new->p_sdist + k) = *(p_pv->p_beg->p_dist + k) + s;
      *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
      *(p_new->p_star + k) = FALSE;
      if (k < p_pv->p_beg->no_pos - 1)
         *(p_new->p_locdist + k) = *(p_pv->p_beg->p_dist + k)
            - *(p_pv->p_beg->p_dist + k + 1);
      else
         *(p_new->p_locdist + k) = *(p_pv->p_beg->p_dist + k);
   }

   p_i = p_pv->p_intv;                  /* for ea proportion list before 1st  */
   while (p_i && !p_i->selectd)         /* sel marker, sets pos to marker pos */
   {                                    /* and to interval positions          */
      *(p_new->p_sdist + k) = s;        /* first, set marker-derived test pos */
      *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
      *(p_new->p_star + k) = TRUE;
      s = p_l->p_next->sel_dist;
      ++k;

      for (j = 0; j < p_i->no_prop; ++j, ++k) /* now set intv prop test pos   */
      {
         *(p_new->p_sdist + k) = s + (1. - *(p_i->p_prop + j))
            * p_l->p_next->dist;
         *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
         *(p_new->p_locdist + k - 1) = *(p_new->p_sdist + k - 1)
            - *(p_new->p_sdist + k);
         *(p_new->p_star + k) = FALSE;
      }
      *(p_new->p_locdist + k - 1) = *(p_new->p_sdist + k - 1) - s;

      p_i = p_i->p_next;
      p_l = p_l->p_next;
   }
   p_new->sel_map_mark = p_l->no;

   for (i = 1; i < p_pm->no_mark_sel; ++i) /* INTERVALS: for ea interval      */
   {                                    /* between selected markers, counts   */
      pi_sav = p_i;                     /* no. positions and allocs list      */
      p_new = p_new->p_next;
      p_new->no_pos = p_i->no_prop;
      while ((p_i = p_i->p_next) && !p_i->selectd)
         p_new->no_pos += p_i->no_prop + 1;
      p_new->sel_map_mark = p_l->no;

      alloc_sintl (p_new);

      p_i = pi_sav;
      s = 0.;
      k = 0;

      do                                /* for this prop list, sets test pos  */
      {
         for (j = 0; j < p_i->no_prop; ++j, ++k)
         {
            *(p_new->p_sdist + k) =
               s + *(p_i->p_prop + j) * (p_l->p_next->dist);
            *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
            if (j == 0)
               *(p_new->p_locdist + k) = *(p_i->p_prop) * (p_l->p_next->dist);
            else
               *(p_new->p_locdist + k) = *(p_new->p_sdist + k)
                  - *(p_new->p_sdist + k - 1);
            *(p_new->p_star + k) = FALSE;
         }
         p_l = p_l->p_next;
         p_i = p_i->p_next;
         s = p_l->sel_dist;
         if (p_new->no_pos)             /* FIXME:  check this addition **** */
         {
            *(p_new->p_locdist + k) = s - *(p_new->p_sdist + k - 1);
            if (p_i && !p_i->selectd)   /* sets mark-derived (unsel) test pos */
            {
               *(p_new->p_sdist + k) = s;
               *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
               *(p_new->p_star + k) = TRUE;
               ++k;
            }
         }
      }
      while (p_i && !p_i->selectd);
   }

   p_new = p_new->p_next;               /* ENDING: counts no. positions       */
   pi_sav = p_i;                        /* and allocates list                 */
   p_new->no_pos = 0;

   if (p_l->no < p_pm->no_mark_map)
      while (p_i)
      {
         p_new->no_pos += p_i->no_prop + 1;
         p_i = p_i->p_next;
      }
   p_new->no_pos += p_pv->p_end->no_pos;
   p_new->sel_map_mark = p_l->no;

   alloc_sintl (p_new);

   p_i = pi_sav;
   k = 0;
   s = 0.;
   while (p_i)                          /* for ea remaining prop list between */
   {                                    /* non-selectd markers,               */
      for (j = 0; j < p_i->no_prop; ++j, ++k) /* first sets test positions    */
      {
         if (j < p_i->no_prop)
         {
            *(p_new->p_sdist + k) = s + *(p_i->p_prop + j) * p_l->p_next->dist;
            *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
         }
         else
         {
            s = p_l->p_next->sel_dist;
            *(p_new->p_sdist + k) = s;
            *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
            *(p_new->p_star + k + 1) = TRUE;
         }
         if (k == 0)
            *(p_new->p_locdist + k) = *(p_new->p_sdist + k);
         else
            *(p_new->p_locdist + k) = *(p_new->p_sdist + k)
               - *(p_new->p_sdist + k - 1);
         *(p_new->p_star + k) = FALSE;
      }
      s = p_l->p_next->sel_dist;        /* then sets marker-derived test pos  */
      *(p_new->p_sdist + k) = s;
      *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
      *(p_new->p_locdist + k) = *(p_new->p_sdist + k)
         - *(p_new->p_sdist + k - 1);
      *(p_new->p_star + k) = TRUE;
      ++k;

      p_l = p_l->p_next;
      p_i = p_i->p_next;
   }

   for (j = 0; j < p_pv->p_end->no_pos; ++j, ++k) /* sets pos after last mark */
   {
      *(p_new->p_sdist + k) = s + *(p_pv->p_end->p_dist + j);
      *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
      if (j == 0)
         *(p_new->p_locdist + k) = *(p_pv->p_end->p_dist);
      else
         *(p_new->p_locdist + k) = *(p_pv->p_end->p_dist + j)
            - *(p_pv->p_end->p_dist + j - 1);
      *(p_new->p_star + k) = FALSE;
   }
   return;
}

PSMAP_PTR alloc_sintv (int i)
{
   PSMAP_PTR p;

   p = malloc (sizeof (PSMAP));
   alloc_chk (WHERE, p);
   p->intv_no = i;
   p->p_sdist = NULL;
   p->p_rf = NULL;
   p->p_locdist = NULL;
   p->p_star = NULL;
   p->p_next = NULL;

   return p;
}

void alloc_sintl (PSMAP_PTR p_si)
{

   if (!p_si->no_pos)
      return;

   p_si->p_sdist = malloc (p_si->no_pos * sizeof (double));
   p_si->p_rf = malloc (p_si->no_pos * sizeof (double));
   p_si->p_locdist = malloc ((p_si->no_pos + 1) * sizeof (double)); /*  + 1   */
   p_si->p_star = malloc ((p_si->no_pos + 1) * sizeof (int)); /*        + 1   */

   return;
}

/**
 * checks the beginning and ending test tloc positions for consistency.
 */
int test_tloc_vext_sx (SIM_PTR p_sel)
{
   if (first_vpmap[FEM]->p_beg->no_pos != first_vpmap[MAL]->p_beg->no_pos)
      err_typ = ERA_WDX;
   else if (first_vpmap[FEM]->p_end->no_pos != first_vpmap[MAL]->p_end->no_pos)
      err_typ = ERA_WDX;
   if (err_typ)
   {
      for_chrom (p_sel->chrom_no);
      return FALSE;
   }
   return TRUE;
}

/**
 * checks the tloc positions in the marker intervals for consistency 
 */
int test_tloc_vint_sx (SIM_PTR p_sel)
/*                                      * checks intra-interval tloc positions
 *                                      * for consistency                     */
{
   PIMAP_PTR p_f, p_m;

   p_f = first_vpmap[FEM]->p_intv;
   p_m = first_vpmap[MAL]->p_intv;

   while (p_f)
   {
      if (p_f->no_prop != p_m->no_prop)
         err_typ = ERA_WIP;
      p_f = p_f->p_next;
      p_m = p_m->p_next;
   }
   if (err_typ)
   {
      for_chrom (p_sel->chrom_no);
      return FALSE;
   }
   return TRUE;
}

/**
 * checks the rules for the external and interval test positions
 * for the program.
 */
int test_vpos_rules (void)
{

   int   it = 0;                        /* no. intervals between sel markers  */
   int   nx = 0;                        /* no. begin or ending test pos lists */
   int   ni = 0;                        /* no. test intv with test pos lists  */
   PSMAP_PTR p_si;

   p_si = first_vpmap[FEM]->p_sintv;

   if (p_si->no_pos != 0)               /* checks beginning                   */
      ++nx;

   while (p_si->p_next->p_next)         /* checks intervals                   */
   {
      p_si = p_si->p_next;
      ++it;
      if (p_si->no_pos)
         ++ni;
   }
   if (p_si->p_next->no_pos != 0)       /* checks ending                      */
      ++nx;

   if (LM_LODS || LM_SCHNELL)           /* tests for no test positions        */
      if (!(nx + ni))
         err_typ = ERA_TPM;             /* FIXME when tloc implemented, test  */

   if (LM_BAYES)                        /* tests for not all intv with prop   */
      if (ni < it)
         err_typ = ERA_IPM;

   if (LM_MARKERS)                      /* FIXME when tloc implemented, test  */
      if (!(nx + ni))
      {
         if (TraitTyp == GENOTYPIC)
            err_typ = ERA_GMP;
         else
            err_typ = ERA_NMP;
      }

   if (err_typ)
   {
      err_txt = ProgNam;
      return FALSE;
   }
   return TRUE;
}

/**
 * allocates global arrays for trial tloc positions.
 *    IntVal:   no. tloc pos + 2
 *    RecVal:   2 sexes, no. tloc pos + 2,  recomb fracts
 *    NTPosInt: no. intervals (no. markers + 1, for beginning, 
 *                             between markers and ending)
 */
void alloc_intval (void)
{
   int   sx;

   RecVal = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, RecVal);

   for (sx = FEM; sx <= MAL; ++sx)
   {
      RecVal[sx] = malloc ((NTPos + 2) * sizeof (double));
      alloc_chk (WHERE, RecVal[sx]);
   }
   IntVal = malloc ((NTPos + 2) * sizeof (int));
   alloc_chk (WHERE, IntVal);

   NTPosInt = malloc ((NMarks + 1) * sizeof (int));
   alloc_chk (WHERE, NTPosInt);

   return;
}

/**
 * stores marker and tloc test positions.
 *    NLocPos - total no. of marker and test tloc positions
 *    LocPos  - data structure containing locus type, interval no.
 *              marker or tloc no., recomb fract, distance from
 *              marker 1
 */
void mak_locpos (void)
{
   int   sx, i, tno, entry;
   double dm[2];                        /* distance to current selectd marker */
   PSMAP_PTR p_tr[2];
   PRDIST_PTR p_mk[2];
   LOCPOS_PTR p_l;

   NLocPos = NTPos + NMarks;

   p_l = malloc (NLocPos * sizeof (LOCPOS));
   alloc_chk (WHERE, p_l);
   LocPos = p_l;

   for (sx = FEM; sx <= MAL; ++sx)
   {
      p_tr[sx] = first_vpmap[sx]->p_sintv;
      entry = FALSE;
      p_mk[sx] = nxt_prntmap (&entry, sx)->p_locus;
      dm[sx] = p_mk[sx]->m1_dist;
   }
   tno = first_vpmap[FEM]->no;

   while (p_tr[FEM])
   {
      for (i = 0; i < p_tr[FEM]->no_pos; ++i) /* TEST positions (intv or ext) */
      {
         p_l->intv_no = p_tr[FEM]->intv_no; /* for this intv, ea test pos, sx */
         p_l->no = tno;                 /* stores recomb fracts and positions */

         if (*(p_tr[FEM]->p_star + i))  /* checks for tloc or unselctd marker */
            p_l->locus_typ = UNSELMRK_LOC;
         else
            p_l->locus_typ = TLOC_LOC;

         for (sx = FEM; sx <= MAL; ++sx)
         {
            if (p_l->intv_no == 0)
            {
               p_l->rf[sx] = .5;
               p_l->rfnxt[sx] = *(p_tr[sx]->p_rf + i);
               p_l->pos[sx] = dm[sx] - *(p_tr[sx]->p_sdist + i);
            }
            else
            {
               p_l->rf[sx] = *(p_tr[sx]->p_rf + i);
               if (p_mk[FEM])
                  p_l->rfnxt[sx] =
                     ((dist2rf (p_mk[sx]->sel_dist)) - p_l->rf[sx])
                     / (1.0 - 2. * p_l->rf[sx]);
               else
                  p_l->rfnxt[sx] = .5;
               p_l->pos[sx] = dm[sx] + *(p_tr[sx]->p_sdist + i);
            }
         }
         ++p_l;
      }
      for (sx = FEM; sx <= MAL; ++sx)
         p_tr[sx] = p_tr[sx]->p_next;

      if (p_mk[FEM])
      {
         p_l->locus_typ = MARK_LOC;     /* selected MARKER postion            */
         p_l->intv_no = p_mk[FEM]->selectd;
         p_l->no = p_mk[FEM]->no;       /* stores recomb fract & pos, ea sex  */
         for (sx = FEM; sx <= MAL; ++sx)

         {
            if (p_mk[sx]->selectd == 1) /* tests for 1st selected marker      */
            {
               p_l->rf[sx] = .5;
               p_l->pos[sx] = dm[sx];
            }
            else
            {
               p_l->rf[sx] = dist2rf (p_mk[sx]->sel_dist);
               dm[sx] += p_mk[sx]->sel_dist;
               p_l->pos[sx] = dm[sx];
            }
            if (p_mk[sx]->p_next)       /* tests for not last selected marker */
               p_l->rfnxt[sx] = dist2rf (p_mk[sx]->p_next->sel_dist);
            else
               p_l->rfnxt[sx] = .5;
            p_mk[sx] = p_mk[sx]->p_next;
         }
         ++p_l;
      }
   }
   return;
}

/**
 * stores trial tloc positions.
 *    NTPosInt[] - list of no. tloc positions in ea marker interval
 *    IntVal[]   - the interval no. corresponding to ea tloc pos.
 */
void mak_intval (void)
{
   int   i, j, ix;
   PSMAP_PTR p_s;

   p_s = first_vpmap[FEM]->p_sintv;
   IntVal[0] = 0;
   ix = 1;

   for (j = 0; j <= NMarks && p_s; ++j)
   {
      NTPosInt[j] = p_s->no_pos;        /* sets no. positions in ea interval  */
      for (i = 0; i < p_s->no_pos; ++i, ++ix)
         IntVal[ix] = j;                /* sets interval for ea tloc position */
      p_s = p_s->p_next;
   }
   IntVal[NTPos + 1] = 0;

   return;
}

/**
 * stores trial tloc recombinaton fractions.
 * makes RecVal[][] - sex-specific recombination fracts corresp to IntVal[]
*/
void mak_recval (void)
{
   PSMAP_PTR p_s;
   int   sx, i, j, ix;

   for (sx = FEM; sx <= MAL; ++sx)
   {
      p_s = first_vpmap[sx]->p_sintv;
      RecVal[sx][0] = .5;               /* sets rec fracts for start of chrom */

      ix = 1;
      for (j = 0; j <= NMarks && p_s; ++j) /* and for ea marker interval      */
      {
         for (i = 0; i < p_s->no_pos; ++i, ++ix)
            RecVal[sx][ix] = *(p_s->p_rf + i);
         p_s = p_s->p_next;
      }
      RecVal[sx][NTPos + 1] = .5;       /* and for end of chrom               */
   }
   return;
}

#if 0
void prnt_atz_pars (void)
{
   int   n;

   if (IBDDROP)
   {
      fprintf (stdout, " Number of MC iterations requested:  %d\n\n", NIt);
      return;
   }

   if (LM_MAP || LM_PVAL || LM_IBDTESTS || LM_AUTO
       || LM_LODS || LM_BAYES || LM_MARKERS || LM_SCHNELL || LM_TWOQTL)
   {
      if (SmplSetup == INDEP_LOC)
         fprintf (stdout, " Locus-by-locus sampling to be used for setup\n\n");
      else if (SmplSetup == SEQIMP)
         fprintf (stdout, " Sequential imputation to be used for setup,"
                  " using %d samples \n\n", NSeqImp);
      if (PeelCmpnt)
         fprintf (stdout, " Peeling is to be by pedigree component\n\n");

      if (GlobalMcmc)
         fprintf (stdout,
                  " Lod scores for all components are estimated globally\n\n");

      if (Scan)
      {
         fprintf (stdout, " Sampling is to be by scan");
         if (ScorIt == UNDEF)
            fprintf (stdout, "\n");
         else if (ScorIt == 1)
            fprintf (stdout, ", with scores estimated every scan\n");
         else
            fprintf (stdout, ", with scores estimated every %d scans\n",
                     ScorIt);
      }
      else
      {
         fprintf (stdout, " Sampling is to be by single step");
         if (ScorIt == UNDEF)
            fprintf (stdout, "\n");
         if (ScorIt == 1)
            fprintf (stdout, ", with scores estimated every step\n");
         else
            fprintf (stdout, ", with scores estimated every %d steps\n",
                     ScorIt);
      }
      if (LM_BAYES && SeqImpIt)
         fprintf (stdout, "         and a restart configuration proposed"
                  " by sequential imputation\n"
                  "         every %d iterations\n\n", SeqImpIt);
      else
         fprintf (stdout, "\n");

      fprintf (stdout, " L-sampler probability set to %.3f\n\n", LSamplProb);

      if (NBrnIt != UNDEF)
      {
         fprintf (stdout, " The number of MC burn-in iterations is %d\n",
                  NBrnIt);
         if (LM_BAYES)
            if (!InPseudo)
               fprintf (stdout,
                        " and the number of pseudo-prior iterations is"
                        " %d\n", NPseudoPrIt);
            else
               fprintf (stdout, "\n Preliminary MCMC iterations to compute"
                        " pseudo-priors are skipped,\n"
                        " as pseudo-priors are input\n");
         fprintf (stdout, "\n");
      }
   }
   if (!LM_MAP && NIt != UNDEF)
   {
      fprintf (stdout, " Number of MC iterations requested is %d\n", NIt);
      if (ChkIt != UNDEF)
         fprintf (stdout, " with progress checked every %d iterations\n",
                  ChkIt);
   }

   if (LM_MARKERS)
      fprintf (stdout,
               " \n Number of batches for MC variance estimation is %d\n",
               NBtch);

   fprintf (stdout, "\n");

   return;
}
#endif

/**
 * prints map showing trial tloc positions.
 * 5 (6, if sex-specific maps) print lines are used for ea chromosome segment,
 * showing the tloc * positions at the start of the chromosome,
 * in ea interval between * markers, and at the end of the chromosome.
 * (a segment consists of as many loci as will fit on the line)
 */
void prnt_vmap (int nmarks, int sxsp_mp)
{
   PSMAP_PTR p_si[3];
   PRNTVMAP_PTR p_vmap;
   double *p_d[3];
   int  *p_f;
   int   sex, i, j, no_loci;

   for (sex = FEM; sex <= AVG; ++sex)
      if (!(p_vmap = first_vpmap[sex]))
         return;                        /* unexpected                         */
      else
      {
         p_si[sex] = p_vmap->p_sintv;
         p_d[sex] = p_si[sex]->p_locdist;
      }
   p_f = p_si[AVG]->p_star;

   /*                                    * first prints the heading           */
   fprintf (stdout, "\n T%d test positions (cM):\n\n", p_vmap->no);

   /*                                    * prints out the BEGINNING positions */
   if (p_si[AVG]->no_pos)               /* (before first selected marker)     */
   {
      j = 0;
      prntvmap_tl0 (*p_f);              /* prints 1st T                       */

      while (j < p_si[AVG]->no_pos)
      {
         no_loci = 1;

         while (j < p_si[AVG]->no_pos && no_loci < max_line_loc)
         {                              /* adds next locus for this line      */
            if (j < p_si[AVG]->no_pos - 1)
               prntvmap_tl (no_loci, *(p_f + 1)); /* next T                   */
            else
               prntmap_mrk (no_loci, p_si[AVG]->sel_map_mark); /* or next M   */
            for (sex = FEM; sex <= AVG; ++sex)
            {
               if ((sxsp_mp && (sex == FEM || sex == MAL))
                   || (!sxsp_mp && sex == AVG))
                  prntmap_dist (sex, no_loci, *p_d[sex]); /* dist- AVG F or M */
               ++p_d[sex];
            }
            ++j;
            ++p_f;
            ++no_loci;
         }
         prntmap_line (no_loci, sxsp_mp); /* prints line for this map segment */
         prntmap_tl0b ();               /* inits next segment                 */
      }
   }

   for (i = 1; i < nmarks; ++i)         /* now prints INTERVAL test positions */
   {                                    /* (between selected markers)         */
      j = 0;
      for (sex = FEM; sex <= AVG; ++sex)
      {
         p_si[sex] = p_si[sex]->p_next;
         p_d[sex] = p_si[sex]->p_locdist;
      }
      p_f = p_si[AVG]->p_star;
      prntmap_mrk0 (p_si[AVG]->sel_map_mark); /* next M                       */

      while (j < p_si[AVG]->no_pos)
      {
         no_loci = 1;

         while (j < p_si[AVG]->no_pos && no_loci < max_line_loc)
         {                              /* adds next locus for this line      */
            if (j < p_si[AVG]->no_pos)
               prntvmap_tl (no_loci, *p_f); /* next T                         */
            for (sex = FEM; sex <= AVG; ++sex)
            {
               if ((sxsp_mp && (sex == FEM || sex == MAL))
                   || (!sxsp_mp && sex == AVG))
                  prntmap_dist (sex, no_loci, *p_d[sex]); /* dist- AVG F or M */
               ++p_d[sex];
            }
            ++j;
            ++p_f;
            ++no_loci;
         }
         if (j == p_si[AVG]->no_pos)
         {
            prntmap_mrk (no_loci, p_si[AVG]->p_next->sel_map_mark); /* next M */
            for (sex = FEM; sex <= AVG; ++sex)
               if ((sxsp_mp && (sex == FEM || sex == MAL))
                   || (!sxsp_mp && sex == AVG))
                  prntmap_dist (sex, no_loci, *p_d[sex]); /* dist- AVG F or M */
            ++no_loci;
         }
         prntmap_line (no_loci, sxsp_mp); /* prints out line for this segment */
         prntmap_tl0b ();               /* inits next segment                 */
      }
   }

   for (sex = FEM; sex <= AVG; ++sex)
   {
      p_si[sex] = p_si[sex]->p_next;
      p_d[sex] = p_si[sex]->p_locdist;
   }
   p_f = p_si[AVG]->p_star;

   if (p_si[AVG]->no_pos)               /* and prints out the ENDING pos      */
   {                                    /* (after last selected marker)       */
      j = 0;
      prntmap_mrk0 (p_si[AVG]->sel_map_mark); /* last M                       */

      while (j < p_si[AVG]->no_pos)
      {
         no_loci = 1;

         while (j < p_si[AVG]->no_pos && no_loci < max_line_loc)
         {                              /* adds next locus for this line      */
            prntvmap_tl (no_loci, *p_f); /* next T                            */
            for (sex = FEM; sex <= AVG; ++sex)
            {
               if ((sxsp_mp && (sex == FEM || sex == MAL))
                   || (!sxsp_mp && sex == AVG))
                  prntmap_dist (sex, no_loci, *p_d[sex]); /* dist- AVG F or M */
               ++p_d[sex];
            }
            ++j;
            ++p_f;
            ++no_loci;
         }
         prntmap_line (no_loci, sxsp_mp); /* prints out line for this segment */
         prntmap_tl0b ();               /* inits next segment                 */
      }
   }
   return;
}

/**
 * looks for chromosome maps, or unlinked spec, for the requested tlocs
 * (tlocs in the trait/tloc list which are marked as selected).
 * marks these tlocs in the test position maps as selected.
 * checks for just one location for each tloc:
 *   if a tloc does not have a map, it is an error.
 *   if a tloc is mapped on more than one chromosome, it is an error. 
 *
 *      for ea trait
          for ea sel trait
            for ea tloc for the trait
              for ea gender
                for ea chrom tloc test pos map
                  for ea mapd tloc in list
                    tloc nos match?
                      set mapd tloc as selctd
                      is found mapd tloc flag set? 
                        n - set found mapd tloc flag
                            set chrom no. found to map chrom no. 
                        y - is chrom no. found = map chrom no?
                            n - error         (mapd on more than 1 chrom)
            if !found (map not present)
              error
*/
int find_tr_tloc_vmap (void)
{
   int   entry, i, tlno;
   TRAITTLOC_PTR p_tt;

   entry = FALSE;                       /* for ea trait/tloc in list:         */
   while ((p_tt = nxt_tr_tl (&entry)) && !err_typ) /* checks trait for selctd */
      if (p_tt->selectd)
         for (i = 0; i < p_tt->no_loci; ++i)
         {
            tlno = *(p_tt->p_tloc + i); /* for each tloc for the trait:       */
            find_nxt_tl_vmap (tlno);    /* look for its map                   */
         }

   if (err_typ)
      return FALSE;
   else
      return TRUE;
}

/**
 * looks for the test position map for the specified tloc
 */
void find_nxt_tl_vmap (tln)
{
   int   entry, sx, found, chrom_no;
   VMAP_PTR p_vmap;

   chrom_no = 0;
   found = 0;
   for (sx = FEM; sx <= AVG; ++sx)      /* for each gender                    */
   {
      entry = FALSE;                    /* for each tloc map                  */
      while ((p_vmap = nxt_tl_vmap (&entry, sx)) && !err_typ)
      {
         if (p_vmap->no == tln)         /* found tloc                         */
         {
            p_d->selectd = TRUE;
            if (!found)
            {
               found = TRUE;
               chrom_no = p_vmap->chrom_no;
            }
            else if (chrom_no != p_vmap->chrom_no)
            {                           /* checks for tloc on > 1 chrom       */
               err_typ = ERD_MCT;
               sprintf (errnumb, "%d", tln);
               err_txt = errnumb;
            }
         }
      }
   }
   if (!found)                          /* checks for not mapped              */
   {
      err_typ = ERD_TNF;
      sprintf (errnumb, "%d", tln);
      err_txt = errnumb;
   }
   return;
}

/**
 * for this chromosome and gender, puts tloc test positions into "print map" 
 * maps not present default to sex-averaged map, then F map
 */
int mak_tlocvprmap (SIM_PTR p_s, PRNTMAP_PTR p_m, int sex)
{
   int   entry;
   VMAP_PTR p_v;
   PRNTVMAP_PTR p_vm;

   entry = FALSE;
   while (p_v = nxt_tl_vmap (&entry, sex)) /* uses deflt rules for gender     */
      if (p_v->selectd)
      {
         p_vm = add_vprntmap (p_v, sex); /* adds tloc positions               */
         add_tloc_vext (p_vm, sex);     /*  for start and end of chrom        */
         add_tloc_vint (p_vm, p_m, sex); /* and for marker intervals          */
         sel_tloc_vpos (p_vm, p_m);     /*  now condenses tloc maps for sel   */
         if (sex == AVG)
         {                              /* tests for the same no. of F & M    */
            if (!test_tloc_vext_sx (p_s)) /* pos at start and end of chrom    */
               if (proc_err ())
                  return FALSE;
            if (!test_tloc_vint_sx (p_s)) /* and in ea interval               */
               if (proc_err ())
                  return FALSE;
            if (!test_pos_rules ())     /* checks rules for test postions     */
               if (proc_err ())
                  return FALSE;
         }
      }
   return;
}

/**
 * prints allele frequencies for the tlocs to be simulated
 * (NOTE:  this is to be expanded, in the future for more than one tloc)
 */
void prnt_vtloc_freqs (PRNTVMAP_PTR p_vmap)
{
   char  prnt_lin[81] = "";

   if (!p_vmap)
      return;

   fprintf (stdout, " Tloc allele frequencies:\n\n"); /* fmts lods tloc freqs */
   fmt_tloc_freqs (prnt_lin, 0, p_vmap->no);

   fprintf (stdout, "%s\n\n", prnt_lin); /* prints the line                   */
   fflush (stdout);

   return;
}
