/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/*
 * the following functions are called by proc_drop(); 
 * some are also called by proc_auto().
 */

/**
 * gets the next simulated chromosome.
 * returns a pointer to the next sim node.
 */
SIM_PTR nxt_sim_chrom (int *p_entry)
{
   static SIM_PTR p_sav;
   if (!*p_entry)
   {
      *p_entry = TRUE;
      return (p_sav = first_sim_chrom_node);
   }
   else
      return (p_sav = p_sav->p_next);
}

/**
 * returns the pointer to the next trait/tloc node
 */
TRAITTLOC_PTR nxt_tr_tl (int *p_entry)
{
   static TRAITTLOC_PTR p_sav;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      return (p_sav = first_trait_tloc_node);
   }
   else if (p_sav = p_sav->p_next)
      return p_sav;
   else
      return NULL;
}

/**
 * returns the pointer to the next inher/tloc node
 */
INHERTLOC_PTR nxt_inh_tl (int *p_entry)
{
   static INHERTLOC_PTR p_sav;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      return (p_sav = first_inher_tloc_node);
   }
   else if (p_sav = p_sav->p_next)
      return p_sav;
   else
      return NULL;
}

/**
 * returns the next no. (trait, tloc or inheritance)
 * in the "simulate/select trait" list.
 * (nos in numerical order)
 */
int nxt_sim_tr_no (int *p_entry)
{
   static int no_tr;
   static int *p_sav;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      if (sim_traits)
      {
         p_sav = sim_traits->p_trait;
         no_tr = sim_traits->no_traits;
         return *p_sav;
      }
      else
         return 0;
   }
   else if (--no_tr)
      return *(++p_sav);
   else
      return 0;
}

/**
 * gets the next tloc from the unlnkd_map_node distance list.
 */
UDIST_PTR nxt_unln_dist (int *p_entry)
{
   static int no_loci;
   static UDIST_PTR p_sav;

   if (!unlnkd_tloc_map_node)           /* tests for unlinked loci present    */
      return NULL;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      if (unlnkd_tloc_map_node)
      {
         no_loci = unlnkd_tloc_map_node->no_loci;
         p_sav = unlnkd_tloc_map_node->p_udist;
         return p_sav;                  /* returns the first tloc             */
      }
      else
         return NULL;
   }
   else if (--no_loci)
      return ++p_sav;                   /* return the next tloc               */
   else
      return NULL;
}

/**
 * returns a pointer to the tloc map for the next chromosome
 * for the specified gender.
 */
MAP_PTR nxt_tl_map (int *p_entry, int sx)
{
   static MAP_PTR p_sav;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      p_sav = first_tloc_map_node[sx];
   }
   else
      p_sav = p_sav->p_next;

   return p_sav;
}

/**
 * returns a pointer to the marker map for the next chromosome
 * for the specified gender.
 */
MAP_PTR nxt_mark_map (int *p_entry, int sx)
{
   static MAP_PTR p_sav;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      p_sav = first_mark_map_node[sx];
   }
   else
      p_sav = p_sav->p_next;

   return p_sav;
}

/**
 * returns a pointer to the next dist list for the specified marker or tloc map.
 */
DIST_PTR nxt_dist (int *p_entry, MAP_PTR p_m)
{
   static int no_loci;
   static DIST_PTR p_sav;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      no_loci = p_m->no_loci;
      return (p_sav = p_m->p_dist);
   }

   else if (--no_loci)
      return ++p_sav;

   return NULL;
}

/**
 * returns the tloc no. of the next mapped tloc to be simulated
 */
int nxt_sim_mapd_tl_no (int *p_entry, SIM_PTR p_sim)
{
   static int no_loci;
   static int *p_sav;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      if (!(no_loci = p_sim->no_tloc_sel))
         return 0;
      else
      {
         --no_loci;
         return (*(p_sav = p_sim->p_tloc));
      }
   }
   else if (no_loci--)
      return (*(++p_sav));
   else
      return 0;
}

/**
 * returns the tloc no. of the next unlinked tloc to be simulated
 */
int nxt_sim_unln_tl_no (int *p_entry)
{
   static int no_loci;
   static int *p_sav;

   if (!sim_unlnkd_tlocs)
      return 0;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      if (!(no_loci = sim_unlnkd_tlocs->no_tloc_sel))
         return 0;
      else
      {
         --no_loci;
         return (*(p_sav = sim_unlnkd_tlocs->p_tloc));
      }
   }
   else if (no_loci--)
      return (*(++p_sav));
   else
      return 0;
}

int nxt_sim_mark_no (int *p_entry, SIM_PTR p_sim)
{
   static int mark_no;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      return (mark_no = 1);
   }
   else
   {
      if (mark_no >= p_sim->no_mark_map)
         return 0;
      else
         return ++mark_no;
   }
}

/**
 * returns the next tloc no. in the "tloc list".
 * if not present, returns 0
 */

int nxt_sim_tl_no (int *p_entry)
{
   static TLLIST_PTR p_tl;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      p_tl = first_tllist_node;
   }
   else
      p_tl = p_tl->p_next;;

   if (p_tl)
      return p_tl->no;
   else
      return 0;
}

/**
 * returns the next tloc no. for the specified inheritance;
 * if not present, returns 0
 */
int nxt_inh_tl_no (int *p_entry, int inhno)
{
   static int no_inh;
   static INHERTLOC_PTR p_hsav;
   static int *p_sav;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      p_hsav = first_inher_tloc_node;
      while (p_hsav && p_hsav->inher_no < inhno)
         p_hsav = p_hsav->p_next;
      if (p_hsav)
      {
         p_sav = p_hsav->p_tloc;
         no_inh = p_hsav->no_loci;
         return *p_sav;
      }
      else
         return 0;
   }
   else if (--no_inh)
      return *(++p_sav);
   else
      return 0;
}

/**
 * returns the next tloc no. for the specified trait;
 * if not present, returns 0
 */
int nxt_tr_tl_no (int *p_entry, int trno)
{
   static int no_tl;
   static TRAITTLOC_PTR p_tsav;
   static int *p_sav;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      p_tsav = first_trait_tloc_node;
      while (p_tsav && p_tsav->trait_no < trno)
         p_tsav = p_tsav->p_next;
      if (p_tsav)
      {
         p_sav = p_tsav->p_tloc;
         no_tl = p_tsav->no_loci;
         return *p_sav;
      }
      else
         return 0;
   }
   else if (--no_tl)
      return *(++p_sav);
   else
      return 0;
}

/**
 * gets the next marker dist for the specified chromosome.
 * returns a pointer to the entry for the next marker in the dist list.
 * uses defaults if specified sex not present.
 */
DIST_PTR nxt_sim_mark_dist (int *p_entry, int sx, SIM_PTR p_sim)
{
   int   i, j;
   static int no_loci;
   MAP_PTR p, p_map;
   static DIST_PTR p_sav;

   if (!*p_entry)
   {
      *p_entry = TRUE;

      if (p_sim->no_mark_map == 1)
         return NULL;

      p_map = NULL;

      for (i = 1; !p_map && i <= 4; ++i)
      {
         if (i == 1)                    /* order to check for map presnt      */
            j = sx;
         else if (i == 2)
            j = AVG;
         else if (i == 3)
            j = FEM;
         else if (i == 4)
            j = MAL;

         if (p = first_mark_map_node[j])
         {
            while (p->p_next && p->chrom_no < p_sim->chrom_no)
               p = p->p_next;
            if (p->chrom_no == p_sim->chrom_no)
            {
               no_loci = p->no_loci;
               p_map = p;
            }
         }
      }
      if (!p_map)
         return NULL;
      else
         return (p_sav = p_map->p_dist);
   }
   if (--no_loci)
      return ++p_sav;
   else
      return NULL;
}

/**
 * gets the next required tloc for the chrom.
 * returns a pointer to the entry for the next tloc in the distance list
 * uses defaults if specified sex is not present 
 */
DIST_PTR nxt_sim_tl_dist (int *p_entry, int sx, SIM_PTR p_sim)
{
   int   i, j;
   static int no_loci;
   MAP_PTR p, p_map;
   static DIST_PTR p_sav;

   if (!*p_entry)                       /* tests for next chromosome          */
   {
      *p_entry = TRUE;
      if (p_sim->no_tloc_sel == 0)
         return NULL;

      p_map = NULL;

      for (i = 1; !p_map && i <= 4; ++i)
      {
         if (i == 1)                    /* order to check for map presnt      */
            j = sx;
         else if (i == 2)
            j = AVG;
         else if (i == 3)
            j = FEM;
         else if (i == 4)
            j = MAL;

         if (p = first_tloc_map_node[j])
         {
            while (p->p_next && p->chrom_no < p_sim->chrom_no)
               p = p->p_next;
            if (p->chrom_no == p_sim->chrom_no)
            {
               no_loci = p->no_loci;
               p_map = p;
            }
         }
      }
      if (!p_map)
         return NULL;
      else
         return (p_sav = p_map->p_dist);
   }
   if (--no_loci)                       /* returns ptr to next tloc (if any)  */
      return ++p_sav;
   else
      return NULL;
}

/**
 * returns the locus no. for the tloc to be simulated.
 * locus no's are relative positions on the chromosomes, then unlinked positions
 */
int get_tl_sim_loc_no (int tlno)
{
   int   j;
   RMAP_PTR pR;

   pR = PRecombMap;
   j = 0;

   while (j < NLoci
          && ((pR->recombTyp == FM || pR->recombTyp == OM)
              || (pR->recombTyp != FM && pR->recombTyp != OM
                  && pR->no != tlno)))
      ++j, ++pR;

   return j;
}

PRDIST_PTR nxt_prntmap_locus (int *p_entry, PRNTMAP_PTR p_pmap)
{
   static PRDIST_PTR p_loc;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      p_loc = p_pmap->p_locus;
   }
   else
      p_loc = p_loc->p_next;

   return p_loc;
}

/**
 * returns a pointer to the next printmap node.
 */
PRNTMAP_PTR nxt_prntmap (int *p_entry, int sx)
{
   static PRNTMAP_PTR p_sav[3];
   if (!*p_entry)
   {
      *p_entry = TRUE;
      return (p_sav[sx] = first_prntmap_node[sx]);
   }
   else
      return (p_sav[sx] = p_sav[sx]->p_next);
}
