/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/* These functions are called by the parameter statement functions, appearing
 * in proc_pars_ss.c and named in parsedefs.h
 */

/**
 * tests for chrom no., marker no's out of range.
 * checks no. of markers to be sim/sel correct for program.
 * sorts marker list and checks for duplication of marker no's.
 */
int chk_sim (int c, int m)
{
   int   i, j = 0;
   int   sav, ix = 0;

   sav = err_typ;
   err_typ = 0;

   if (CH == c)
   {
      if (j_par[ix] < 1 || j_par[ix] > psx->max_chrom)
         err_typ = ERD_CHO;
      ++ix;
   }

   if (err_typ)
      return FALSE;

   if (SS_MKALL == m)                   /* does no checking for "all" markers */
      ;
   else if (SS_MKLIST == m)             /* checks marker no's in list         */
   {
      for (i = ix; i < nk_par; ++i, ++ix)
      {
         if (j_par[ix] < 1)
            err_typ = ERD_MKO;
         ++j;
      }
      if (j > psx->max_mark)            /* checks no. markers requested       */
         err_typ = ERD_MKX;

      if (!err_typ)                     /* sorts mark no's & checks for dups  */
      {
         qsort (&(j_par[ix]), nk_par - ix, sizeof (int), qfunc_int);

         for (i = ix; i < nk_par - 1; ++i, ++ix)
            if (j_par[i] == j_par[i + 1])
            {
               err_typ = ERD_DMK;
               sprintf (errnumb, "%d", j_par[i]);
               err_txt = errnumb;
            }
      }
   }
   if (err_typ)
      return FALSE;

   err_typ = sav;
   return TRUE;
}

/**
 * tests for chrom no., trait no's out of range.
 * checks no. of traits to be sim/sel correct for program.
 * sorts trait list and checks for duplication of marker no's.
 */
int chk_sim_tr (int m)
{
   int   i;
   int   sav, ix = 0;

   sav = err_typ;
   err_typ = 0;

   if (SS_TRLIST == m)                  /* checks list of trait no's          */
   {
      if (nk_par > psx->max_trait)
         err_typ = ERD_TRX;
      else
         for (i = 0; i < nk_par; ++i)
            if (j_par[i] < 1 || j_par[i] > MAXTR)
               err_typ = ERD_TRO;
   }
   else if (SS_TLLIST == m)             /* checks list of tloc no's           */
   {
      if (nk_par > psx->max_tloc)
         err_typ = ERD_TRX;
      else
         for (i = 0; i < nk_par; ++i)
            if (j_par[i] < 1 || j_par[i] > MAXTL)
               err_typ = ERD_TRO;
   }
   else if (SS_INHER == m)
   {
      if (nk_par > psx->max_trait)
         err_typ = ERD_HRX;
      else
         for (i = 0; i < nk_par; ++i)
            if (j_par[i] < 1 || j_par[i] > MAXTR)
               err_typ = ERD_IHO;
   }

   if (!err_typ && nk_par > 1)          /* sorts trait no's & checks for dups */
   {
      qsort (&(j_par[0]), nk_par, sizeof (int), qfunc_int);

      for (i = 0; i < nk_par - 1; ++i, ++ix)
         if (j_par[i] == j_par[i + 1])
         {
            if (SS_TRLIST == m)
               err_typ = ERD_DTR;
            else
               err_typ = ERD_DTW;
            sprintf (errnumb, "%d", j_par[i]);
            err_txt = errnumb;
         }
   }

   if (err_typ)
      return FALSE;

   err_typ = sav;
   return TRUE;
}

/**
 * checks for this chromosome already sim/sel,
 * and for markers already specified 
 */

SIM_PTR chk_sim_node (int c, int m)
{
   SIM_PTR p_node;

   int   n = 0;

   if (CH == c)
      n = j_par[0];

   if (p_node = first_sim_chrom_node)
   {
      while (p_node->p_next && p_node->chrom_no < n)
         p_node = p_node->p_next;

      if (p_node->chrom_no == n)
      {
         if (SS_MKALL == m || SS_MKLIST == m)
         {
            if (p_node->no_mark_sel)    /* checks for existing marker list    */
               if (n == 0)
                  err_typ = ERD_RSM;
               else
                  err_typ = ERD_RCM;
         }
         return p_node;
      }
   }
   return NULL;
}

/**
 * checks for sim/sel of traits or of inheritance already specified 
 */

SIMTRAIT_PTR chk_sim_tr_node (void)
{
   SIMTRAIT_PTR p_node;

   if (p_node = sim_traits)
   {
      if (p_node->trait_typ == SS_TRLIST)
         err_typ = ERD_RST;
      else if (p_node->trait_typ == SS_TLLIST)
         err_typ = ERD_RSL;
      else if (p_node->trait_typ == SS_INHER)
         err_typ = ERD_RSI;
   }
   return p_node;
}

/** 
 * checks set tlocs for trait
 */

int chk_set_tr_tl (int t)
{
   int   i, nt, nl;
   int  *tab;

   if (TRLIST_TLLIST == t)              /* gets no. traits and tlocs          */
   {
      nt = nk_par / 2;
      nl = nt;
      if (nt + nl != nk_par)            /* checks same no. traits and tlocs   */
         err_typ = ERD_TTL;
   }
   else if (TR_TLLIST == t)
   {
      nt = 1;
      nl = nk_par - 1;
      if (nl > psx->max_tloc)           /* checks no. of tlocs for trait      */
      {
         err_typ = ERD_MTL;
         sprintf (errnumb, "%d", psx->max_tloc);
         err_txt = errnumb;
      }
   }
   if (err_typ)
      return FALSE;

   for (i = 0; i < nt; ++i)             /* checks trait no's                  */
      if (j_par[i] < 1 || j_par[i] > MAXTR)
      {
         err_typ = ERD_TRO;
         return FALSE;
      }

   for (i = nt; i < nt + nl; ++i)       /* checks tloc no's                   */
      if (j_par[i] < 1 || j_par[i] > MAXTL)
         err_typ = ERD_TLO;

   if (err_typ)
      return FALSE;

   if (nt > 1)                          /* reorders list by trait no's        */
   {
      tab = malloc (nt * 2 * sizeof (int));
      alloc_chk (WHERE, tab);
      for (i = 0; i < nt; ++i)
      {
         tab[2 * i] = j_par[i];
         tab[2 * i + 1] = j_par[nt + i];
      }
      qsort (tab, nt, 2 * sizeof (int), qfunc_int); /* sorts trt or inh no's  */

      for (i = 0; i < nt - 1; ++i)      /* checks for dup trait no's          */
         if (tab[2 * i] == tab[2 * i + 2])
         {
            err_typ = ERD_DTR;
            sprintf (errnumb, "%d", tab[2 * i]);
            err_txt = errnumb;
         }

      for (i = 0; i < 2 * nt; ++i)      /* copies back                        */
         j_par[i] = tab[i];
      free (tab);
   }

   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks for trait/tloc node already present for this trait.
 */
TRAITTLOC_PTR chk_tr_tl_node (int it)
{
   TRAITTLOC_PTR p_node;

   if (p_node = first_trait_tloc_node)
   {
      while (p_node->p_next && p_node->trait_no < j_par[it])
         p_node = p_node->p_next;

      if (p_node->trait_no == j_par[it])
      {
         err_typ = ERD_RTL;
         return p_node;
      }
   }
   return NULL;
}

/** 
 * checks set tlocs for inher
 */

int chk_set_inh_tl (void)
{
   int   i, ni, nl;
   int  *tab;

   ni = nk_par / 2;                     /* gets no. inherits and no. tlocs    */
   nl = ni;
   if (ni + nl != nk_par)               /* checks same no.                    */
   {
      err_typ = ERD_ITL;
      return FALSE;
   }

   for (i = 0; i < ni; ++i)             /* checks inheritance no's            */
      if (j_par[i] < 1 || j_par[i] > MAXTR)
      {
         err_typ = ERD_IHO;
         return FALSE;
      }

   for (i = ni; i < ni + nl; ++i)       /* checks tloc no's                   */
      if (j_par[i] < 1 || j_par[i] > MAXTL)
      {
         err_typ = ERD_TLO;
         return FALSE;
      }

   if (ni > 1)                          /* reorders list by inheritance no's */
   {
      tab = malloc (ni * 2 * sizeof (int));
      alloc_chk (WHERE, tab);
      for (i = 0; i < ni; ++i)
      {
         tab[2 * i] = j_par[i];
         tab[2 * i + 1] = j_par[ni + i];
      }
      qsort (tab, ni, 2 * sizeof (int), qfunc_int);

      for (i = 0; i < ni - 1; ++i)      /* checks for dup inheritance no's   */
         if (tab[2 * i] == tab[2 * i + 2])
         {
            err_typ = ERD_DTR;
            sprintf (errnumb, "%d", tab[2 * i]);
            err_txt = errnumb;
         }
      for (i = 0; i < 2 * ni; ++i)      /* copies back                        */
         j_par[i] = tab[i];
      free (tab);
   }

   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks for inher/tloc node already present for this inheritance no.
 */
INHERTLOC_PTR chk_inh_tl_node (int it)
{
   INHERTLOC_PTR p_node;

   if (p_node = first_inher_tloc_node)
   {
      while (p_node->p_next && p_node->inher_no < j_par[it])
         p_node = p_node->p_next;

      if (p_node->inher_no == j_par[it])
      {
         err_typ = ERD_RIL;
         return p_node;
      }
   }
   return NULL;
}

/**
 *  free list of markers to simulate or select
 */
void free_mark_list (SIM_PTR p_node)
{
   if (p_node && p_node->p_mark)
   {
      free (p_node->p_mark);
      p_node->p_mark = NULL;
      p_node->no_mark_sel = 0;
   }
   return;
}

/**
 *  free list of traits for simulation or selection
 */
void free_trait_list (SIMTRAIT_PTR p_node)
{
   if (p_node && p_node->p_trait)
   {
      free (p_node->p_trait);
      p_node->p_trait = NULL;
      p_node->no_traits = 0;
   }
   return;
}

/**
 * free the list of tlocs for this trait
 */
void free_tr_tl_list (TRAITTLOC_PTR p_t)
{
   p_t->no_loci = 0;
   free (p_t->p_tloc);
   p_t->p_tloc = NULL;

   return;
}

/**
 * free the list of tlocs for this inheritance
 */
void free_inh_tl_list (INHERTLOC_PTR p_t)
{
   p_t->no_loci = 0;
   free (p_t->p_tloc);
   p_t->p_tloc = NULL;

   return;
}

/**
 * allocates the sim node;  inserts in list in chromosome order
 */
SIM_PTR alloc_sim_node (int c)
{
   SIM_PTR new_node, cur_node;
   int   n = 0;

   if (CH == c)
      n = j_par[0];

   new_node = malloc (sizeof (SIM));
   alloc_chk (WHERE, new_node);
   new_node->chrom_no = n;
   new_node->mark_typ = 0;
   new_node->no_mark_sel = 0;
   new_node->no_mark_map = 0;
   new_node->p_mark = NULL;
   new_node->no_tloc_sel = 0;
   new_node->p_tloc = NULL;
   new_node->p_next = NULL;

   if (!first_sim_chrom_node || first_sim_chrom_node->chrom_no > n)
   {
      new_node->p_next = first_sim_chrom_node; /* puts at start of list       */
      first_sim_chrom_node = new_node;  /* (may be only node)                 */
   }
   else
   {
      cur_node = first_sim_chrom_node;  /* inserts node in list               */
      /*                                   or adds to end of list             */
      while (cur_node->p_next && cur_node->p_next->chrom_no < n)
         cur_node = cur_node->p_next;

      new_node->p_next = cur_node->p_next;
      cur_node->p_next = new_node;
   }
   return new_node;
}

/**
 * allocates the trait/tloc/inher sim/sel node
 */
SIMTRAIT_PTR alloc_sim_tr_node (void)
{
   sim_traits = malloc (sizeof (SIMTRAIT));
   alloc_chk (WHERE, sim_traits);
   sim_traits->trait_typ = 0;
   sim_traits->no_traits = 0;
   sim_traits->p_trait = NULL;

   return sim_traits;
}

/**
 *  allocates the list of markers to simulate or select
 */
void alloc_mark_list (SIM_PTR p_node)
{
   p_node->p_mark = malloc (p_node->no_mark_sel * sizeof (int));
   alloc_chk (WHERE, p_node->p_mark);

   return;
}

/**
 *  allocates the list of traits to simulate or select
 */
void alloc_trait_list (SIMTRAIT_PTR p_node)
{
   p_node->p_trait = malloc (p_node->no_traits * sizeof (int));
   alloc_chk (WHERE, p_node->p_trait);

   return;
}

/**
 *  allocate the trait/tloc node for this trait and add to list.
 */
TRAITTLOC_PTR alloc_tr_tl_node (int it)
{
   TRAITTLOC_PTR new_node, cur_node;

   new_node = malloc (sizeof (TRAITTLOC));
   alloc_chk (WHERE, new_node);

   new_node->trait_no = j_par[it];
   new_node->list_typ = 0;
   new_node->selectd = 0;
   new_node->no_loci = 0;
   new_node->p_tloc = NULL;
   new_node->p_next = NULL;

   if (!first_trait_tloc_node || first_trait_tloc_node->trait_no > j_par[it])
   {
      new_node->p_next = first_trait_tloc_node; /* puts at start of list      */
      first_trait_tloc_node = new_node; /* (may be only node)                 */
   }
   else
   {
      cur_node = first_trait_tloc_node; /* inserts in or adds to end of list  */
      while (cur_node->p_next && cur_node->p_next->trait_no < j_par[it])
         cur_node = cur_node->p_next;

      new_node->p_next = cur_node->p_next;
      cur_node->p_next = new_node;
   }
   return new_node;
}

/**
 *  allocates the list of tlocs for the trait
 */
void alloc_tr_tl_list (TRAITTLOC_PTR p_node, int t)
{
   if (TRLIST_TLLIST == t)
      p_node->no_loci = 1;
   else if (TR_TLLIST == t)
      p_node->no_loci = nk_par - 1;

   p_node->p_tloc = malloc (p_node->no_loci * sizeof (int));
   alloc_chk (WHERE, p_node->p_tloc);

   return;
}

/**
 *  allocate the inher/tloc node for this inheritance and add to list.
 */
INHERTLOC_PTR alloc_inh_tl_node (int it)
{
   INHERTLOC_PTR new_node, cur_node;

   new_node = malloc (sizeof (INHERTLOC));
   alloc_chk (WHERE, new_node);

   new_node->inher_no = j_par[it];
   new_node->list_typ = 0;
   new_node->selectd = 0;
   new_node->no_loci = 0;
   new_node->p_tloc = NULL;
   new_node->p_next = NULL;

   if (!first_inher_tloc_node || first_inher_tloc_node->inher_no > j_par[it])
   {
      new_node->p_next = first_inher_tloc_node; /* puts at start of list      */
      first_inher_tloc_node = new_node; /* (may be only node)                 */
   }
   else
   {
      cur_node = first_inher_tloc_node; /* inserts in or adds to end of list  */
      while (cur_node->p_next && cur_node->p_next->inher_no < j_par[it])
         cur_node = cur_node->p_next;

      new_node->p_next = cur_node->p_next;
      cur_node->p_next = new_node;
   }
   return new_node;
}

/**
 *  allocates the list of tlocs for the inher
 */
void alloc_inh_tl_list (INHERTLOC_PTR p_node)
{
   p_node->no_loci = nk_par - 1;

   p_node->p_tloc = malloc (p_node->no_loci * sizeof (int));
   alloc_chk (WHERE, p_node->p_tloc);

   return;
}

/**
 *  sets the marker specification type, chromosome no. and the no. markers
 */
void stor_sim_data (SIM_PTR p_node, int c, int m)
{
   if (CH == c)
      p_node->chrom_no = j_par[0];
   else
      p_node->chrom_no = 0;

   if (SS_MKALL == m || SS_MKLIST == m) /* set sim/sel marker statement type  */
   {
      p_node->mark_typ = m;
      if (SS_MKALL == m)                /* set all markers sim/sel            */
         p_node->no_mark_sel = -1;
      else if (SS_MKLIST == m)          /* or set a list of markers sim/sel   */
         p_node->no_mark_sel = nk_par;
   }

   return;
}

/**
 *  sets the trait/tloc/inher type and the no. of traits or trait loci
 */
void stor_sim_tr_data (SIMTRAIT_PTR p_node, int m)
{
   p_node->trait_typ = m;               /* set sim/sel trait/tloc/inher type  */
   p_node->no_traits = nk_par;

   return;
}

/**
 *  stores the list of sel/sim  markers, sorted by marker no.
 */
void stor_mark_list (SIM_PTR p_node, int c)
{
   int   ix;
   int  *p_m;

   ix = CH == c ? 1 : 0;

   for (p_m = p_node->p_mark; p_m < p_node->p_mark + p_node->no_mark_sel; ++p_m)
      *p_m = j_par[ix++];

   return;
}

/**
 *  stores the list of sel/sim traits, inheritances or tlocs.
 */
void stor_trait_list (SIMTRAIT_PTR p_node)
{
   int   ix = 0;
   int  *p_t;

   for (p_t = p_node->p_trait; p_t < p_node->p_trait + p_node->no_traits; ++p_t)
      *p_t = j_par[ix++];

   return;
}

/**
 *  stores the list of tlocs for the trait.
 */
void stor_tr_tl_list (TRAITTLOC_PTR p_node, int it, int t)
{
   int  *p_t;

   if (TRLIST_TLLIST == t)
      *(p_node->p_tloc) = j_par[it + 1];

   else if (TR_TLLIST == t)             /* FIXME                          */
      for (p_t = p_node->p_tloc; p_t < p_node->p_tloc + p_node->no_loci; ++p_t)
         *p_t = j_par[++it];

   return;
}

/**
 *  stores the list of tlocs for the inheritance.
 */
void stor_inh_tl_list (INHERTLOC_PTR p_node, int it)
{
   *(p_node->p_tloc) = j_par[it + 1];

   return;
}
