/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

int in_ped_fil (void)                   /* input pedigree file                */
{
   if (chk_stat ())
      chk_fil (PED);

   return err_typ;
}

int in_pedsiz (void)                    /* input pedsize statement            */
{
   if (chk_stat ())
   {
      chk_repeat ();
      PedSize = j_par[0];
      if (PedSize < MINPED)
         err_typ = ERR_PED;
      else if (PedSize > pmx->max_ped)
      {
         err_typ = ERR_PEX;
         sprintf (errnumb, "%d", pmx->max_ped);
         err_txt = errnumb;
      }
   }
   return err_typ;
}

int in_ped_int (void)                   /* input pedigree record integer      */
{
   if (chk_stat ())
   {
      err_typ = ERR_PPI;

      chk_repeat ();
      NNams = 3;
      NInts = j_par[0] - NNams;
      if (NInts < 0 || NInts > FMAX_INT)
         err_typ = ERR_PF1;
      NReals = 0;
   }
   return err_typ;
}

int in_ped_int_rl (void)                /* input pedigree record integ real   */
{
   if (chk_stat ())
   {
      err_typ = ERR_PPR;

      chk_repeat ();
      NNams = 3;
      NInts = j_par[0] - NNams;
      NReals = j_par[1];
      if (NInts < 0 || NInts > FMAX_INT)
         err_typ = ERR_PF1;
      if (NReals < 0 || NReals > FMAX_REAL)
         err_typ = ERR_PF2;
   }
   return err_typ;
}

int in_ped_n (void)                     /* input pedigree record names        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      NNams = j_par[0];
      if (NNams != 3)
         err_typ = ERR_PN3;
      else
      {
         NInts = 0;
         NReals = 0;
      }
   }
   return err_typ;
}

int in_ped_n_i (void)                   /* input pedigree record names int    */
{
   if (chk_stat ())
   {
      chk_repeat ();
      NNams = j_par[0];
      if (NNams != 3)
         err_typ = ERR_PN3;
      else
      {
         NInts = j_par[1];
         if (NInts < 0 || NInts > FMAX_INT - NNams)
            err_typ = ERR_PF1;
         NReals = 0;
      }
   }
   return err_typ;
}

int in_ped_n_i_r (void)                 /* input pedigree record nam int real */
{
   if (chk_stat ())
   {
      chk_repeat ();
      NNams = j_par[0];
      if (NNams != 3)
         err_typ = ERR_PN3;
      else
      {
         NInts = j_par[1];
         NReals = j_par[2];
         if (NInts < 0 || NInts > FMAX_INT - NNams)
            err_typ = ERR_PF1;
         if (NReals < 0 || NReals > FMAX_REAL)
            err_typ = ERR_PF2;
      }
   }
   return err_typ;
}

int in_ped_n_r (void)                   /* input pedigree record names real   */
{
   if (chk_stat ())
   {
      chk_repeat ();
      NNams = j_par[0];
      if (NNams != 3)
         err_typ = ERR_PN3;
      else
      {
         NInts = 0;
         NReals = j_par[1];
         if (NReals < 0 || NReals > FMAX_REAL)
            err_typ = ERR_PF2;
      }
   }
   return err_typ;
}

int in_ped_tr_int (void)                /* input pedigree reco traits integrs */
{
   TRTPED_PTR p_trtped;
   int   n, i;

   if (chk_stat ())
   {
      n = nk_par / 2;
      if (chk_trtped (INTDAT))
         for (i = 0; i < n; i++)
         {
            if (!(p_trtped = chk_trtped_node (j_par[i], SS_TRLIST)))
               p_trtped = alloc_trtped_node (j_par[i]);
            stor_trtped_pos (INTDAT, j_par[i], SS_TRLIST, &j_par[n + i],
                             p_trtped);
         }
   }
   return err_typ;
}

int in_ped_tr_int2 (void)               /* input ped reco traits integr pairs */
{
   TRTPED_PTR p_trtped;
   int   n, i;

   if (chk_stat ())
   {
      n = nk_par / 3;
      if (chk_trtped (INTDAT2))
         for (i = 0; i < n; i++)
         {
            if (!(p_trtped = chk_trtped_node (j_par[i], SS_TRLIST)))
               p_trtped = alloc_trtped_node (j_par[i]);
            stor_trtped_pos (INTDAT2, j_par[i], SS_TRLIST, &j_par[n + 2 * i],
                             p_trtped);
         }
   }
   return err_typ;
}

int in_ped_inh_int2 (void)              /* input ped reco inher integer pairs */
{
   TRTPED_PTR p_trtped;
   int   n, i;

   if (chk_stat ())
   {
      n = nk_par / 3;
      if (chk_trtped (INHDAT2))
         for (i = 0; i < n; i++)
         {
            if (!(p_trtped = chk_trtped_node (j_par[i], SS_INHER)))
               p_trtped = alloc_trtped_node (j_par[i]);
            stor_trtped_pos (INHDAT2, j_par[i], SS_INHER, &j_par[n + 2 * i],
                             p_trtped);
         }
   }
   return err_typ;
}

int in_ped_tr_real (void)               /* input pedigree reco traits reals   */
{
   TRTPED_PTR p_trtped;
   int   n, i;

   if (chk_stat ())
   {
      n = nk_par / 2;
      if (chk_trtped (REALDAT))
         for (i = 0; i < n; i++)
         {
            if (!(p_trtped = chk_trtped_node (j_par[i], SS_TRLIST)))
               p_trtped = alloc_trtped_node (j_par[i]);
            stor_trtped_pos (REALDAT, j_par[i], SS_TRLIST, &j_par[n + i],
                             p_trtped);
         }
   }
   return err_typ;
}

int in_eig_fil (void)                   /* input eigenvalue file              */
{
   if (chk_stat ())
      chk_fil (IGEN);

   return err_typ;
}

int in_mrkdat_fil (void)                /* input marker data file             */
{
   if (chk_stat ())
      chk_fil (MDAT);

   return err_typ;
}

int in_peel_fil (void)                  /* input peeling file                 */
{
   if (chk_stat ())
      chk_fil (PEEL);

   return err_typ;
}

int in_resc_fil (void)                  /* input rescue file                  */
{
   if (chk_stat ())
      chk_fil (IRESC);

   return err_typ;
}

int in_seed_fil (void)                  /* input seed file                    */
{
   if (chk_stat ())
      chk_fil (SEED);

   return err_typ;
}

int in_xtra_fil (void)                  /* input extra file                   */
{
   if (chk_stat ())
      chk_fil (XTRA);

   return err_typ;
}

int in_ord_fath (void)                  /* input record father mother         */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (parentOrdIn == MA1ST)
         err_typ = ERR_ORD;
      parentOrdIn = PA1ST;
   }
   return err_typ;
}

int in_ord_moth (void)                  /* input record mother father         */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (parentOrdIn == PA1ST)
         err_typ = ERR_ORD;
      parentOrdIn = MA1ST;
   }
   return err_typ;
}

int in_gend_abs (void)                  /* input record gender absent         */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SexIn == Y)
         err_typ = ERR_GND;
      SexIn = N;
   }
   return err_typ;
}

int in_gend_pres (void)                 /* input record gender present        */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SexIn == N)
         err_typ = ERR_GND;
      SexIn = Y;
   }
   return err_typ;
}

int in_obs_abs (void)                   /* input record observed absent       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (ObsIn == Y)
         err_typ = ERR_OPA;
      ObsIn = N;
   }
   return err_typ;
}

int in_obs_pres (void)                  /* input record observed present      */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (ObsIn == N)
         err_typ = ERR_OPA;
      ObsIn = Y;
   }
   return err_typ;
}

int ignor_gend (void)                   /*  ignore gender                     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (SexOp == ASSIGN)
         err_typ = ERR_IGG;
      SexOp = IGNORE_M;
   }
   return err_typ;
}
