/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/*
 * the following functions are called by proc_lods() and proc_lodsf(). 
 */

/** 
 * gets the requested tloc test position map.
 * NOTE:  needs to be expanded for more than 1 tloc (add entry arg).
 * returns the pointer to the tloc map for the selected chromosome  
 * and gender, using gender defaults
 */
VMAP_PTR nxt_tl_vmap (int sx, SIM_PTR p_sel)
{
   int   i, j;
   VMAP_PTR p, p_vmap = NULL;

   for (i = 1; !p_vmap && i <= 4; ++i)
   {
      if (i == 1)                       /* checks for gender-specific map    */
         j = sx;                        /* present in this order             */
      else if (i == 2)
         j = AVG;
      else if (i == 3)
         j = FEM;
      else if (i == 4)
         j = MAL;

      if (p = first_tloc_vmap_node[j])
      {
         while (p->p_next && (p->chrom_no < p_sel->chrom_no))
            p = p->p_next;
         if (p->chrom_no == p_sel->chrom_no)
            p_vmap = p;
      }
   }
   return p_vmap;
}

/**
 * gets the requested beginning and/or ending map for test tloc positions.
 * returns the pointer to the tloc map for the beginning or end of the
 * chromosome for the specified gender, using gender defaults.
 */
XMAP_PTR nxt_tloc_vext (int sx, PRNTVMAP_PTR p_pr, int d)
{
   int   i, j;
   VMAP_PTR p;
   XMAP_PTR p_x = NULL;

   for (i = 1; !p_x && i <= 4; ++i)
   {
      if (i == 1)                       /* checks for gender-specific map     */
         j = sx;                        /* data present in this order         */
      else if (i == 2)
         j = AVG;
      else if (i == 3)
         j = FEM;
      else if (i == 4)
         j = MAL;
      if (p = first_tloc_vmap_node[j])
      {
         while (p->p_next && (p->chrom_no < p_pr->chrom_no))
            p = p->p_next;
         if (p->chrom_no == p_pr->chrom_no)
            if (d == BDM)
               p_x = p->p_beg;
            else if (d == EDM)
               p_x = p->p_end;
      }
   }
   return p_x;
}

/**
 * gets the requested tloc interval proportions map.
 * returns the pointer to the interval proportions list for the
 * specified interval and gender using gender defaults              
 */
IMAP_PTR nxt_tloc_vint (int sx, int mi, PRNTVMAP_PTR p_pr)
{
   int   i, j;
   VMAP_PTR p;
   IMAP_PTR p_i = NULL;

   for (i = 1; !p_i && i <= 4; ++i)
   {
      if (i == 1)                       /* checks for gender-specific map     */
         j = sx;                        /* data present in this order         */
      else if (i == 2)
         j = AVG;
      else if (i == 3)
         j = FEM;
      else if (i == 4)
         j = MAL;
      if (p = first_tloc_vmap_node[j])
      {                                 /* first finds chrom for this gender  */
         while (p->p_next && (p->chrom_no < p_pr->chrom_no))
            p = p->p_next;
         if (p->chrom_no == p_pr->chrom_no)
         {                              /* then looks for the interval        */
            p_i = p->p_intv;
            while (p_i && p_i->p_next && p_i->intv_no < mi)
               p_i = p_i->p_next;
            if (!(p_i && p_i->intv_no == mi))
               p_i = NULL;
         }
      }
   }
   return p_i;
}
