/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

  /*****************************************************************
   * L-Sample Step     SCH July  1999                              *
   *   to subroutine   EAT March 2000                              *
   *   modified for scan or random locus  EAT Dec 2000             *
   *                                                               *
   * L-peel likelihood compute;  EAT May 2001                      *
   *   rblik() returns log P(Y_T|S_M) for specified trait          *
   *   position; lsample_flag =0; S is NOT resampled               *
   *                                                               *
   *  do_rblik modified for batch-mean computation: 20-02-2002 EAT *
   *                                                               *
   *  do_lsmpl_shnl moved from Schnell/            Apr 09 2004 MAJ *  
   *                                                               *
   *  Routines modified to remove genotypic array y. Sep 2004  EAT *
   *  do_lsmp updated, do_lsmpl_shnl eliminated      Sep 2004 EAT  *
   *                                                               *
   *****************************************************************/
#include "lsample.h"
#include "msample.h"

double do_lsmpl (int nl, int jj, double **prob, int ng)
{
   double z;
   int   err, c;
   set_transmission (loci, jj, nl + 1);
   z = locus_mjrpeel_cmp (0, loci[0].ve, 1.0, 1, 0, NULL, loci + jj);

   /* checks for Zero probability in peeling operation                        */
   if (z == -DBL_MAX)
      give_up_where (WHERE, ERS_zpp);

   set_FGL (jj);

   for (c = 0; c < n_compnts; ++c)
   {
      prob[c][jj] = seg_pen2 (c, loci[jj].freq, jj, ng, &err);
      if (prob[c][jj] <= 0.0)
         give_up_where (WHERE, ERS_zpl, jj);
   }

   return z;
}

void do_rblik (int ntp, int batch, int cmpflag,
               int *lleft, int *lrght, double *rml, double *rfl,
               double *rmr, double *rfr, double **rb, double ***rbb,
               double *bl, double *cl)
/* ntp = # linked trait positions, at intervals loc and recombination rec,
   loc_left, etc, are recombinations, intervals etc, for trait positions.
   rb_prob is cumulative likelihood estimate
   base_lik = base (unlinked) likelihood, computed once only   */
{
   double z;
   int   k, c;

   for (k = 1; k < ntp; ++k)
   {
      if ((lleft[k] != lrght[k]) || (TraitTyp != GENOTYPIC))
/* This check is reinstated, EAT Oct 2005, as it is NECESSARY for some
   computations involving some complex peeling operations if trait-marker
   incompatibility is possible */
/* NOTE: only GENOTYPIC trait case is excluded here: if user is doing
    discrete trait with some penetraces 0, same could happen: warning
    on input needed for those cases ??  */
      {
/*
 *  printf ("\n starting trait position %d", k); fflush(stdout);
 *  printf ("\n locus left %d recom %f %f,  right %d recom %f %f",
 *  lleft[k], rml[k], rfl[k], lrght[k], rmr[k], rfr[k]);
 *  fflush(stdout);
 */
         z = do_mjrlik (lleft[k], lrght[k], rml[k], rfl[k], rmr[k], rfr[k],
                        cmpflag, cl);

         if (cmpflag)
         {
            for (c = 1; c <= n_compnts; ++c)
            {
               rb[c][k] += exp (cl[c] - bl[c]);
               if (rbb)
                  rbb[c][batch][k] += exp (cl[c] - bl[c]);
            }
         }

         rb[0][k] += exp (z - bl[0]);
         if (rbb)
            rbb[0][batch][k] += exp (z - bl[0]);
/*
 *    printf ("\n end of loop for k=%d", k); fflush(stdout);
 *    printf ("   current rb_prob = %e", rb_prob[k]); fflush(stdout);
 */
      }
   }
   return;
}

double do_mjrlik (int ll, int lr, double rml, double rfl,
                  double rmr, double rfr, int cflg, double *clik)
{
   double z;
/*
   transmissions MUST be computed by set_simple_trans ONLY
   lsample_flag  MUST be 0 -- S's are NOT resampled.
 */
   set_simple_trans (loci, ll, lr, rml, rfl, rmr, rfr);

   /* trait locus peel; sample_flag=0   */
   if (TraitTyp == QUANTITATIVE)
      z = locus_mjrpeel_cmp (0, loci[0].ve, 1.0, 0, cflg, clik, loci);
   else
      z = locus_mjrpeel_cmp (0, 1.0, 1.0, 0, cflg, clik, loci);
   /*    printf ("\n completed peel; like = %e",z); fflush(stdout);  */

   /* checks for Zero probability in peeling operation            */
   if (z == -DBL_MAX)
      give_up_where (WHERE, ERS_zpp);

   return (z);
}

void do_fzplik (int t, int ntp,
                int *lleft, int *lrght, double *rml, double *rfl,
                double *rmr, double *rfr, double **scor)
/* ntp = # linked trait positions, at intervals loc and recombination rec,
   loc_left, etc, are recombinations, intervals etc, for trait positions.
   transmissions MUST be computed by set_simple_trans ONLY
   lsample_flag  MUST be 0 -- S's are NOT resampled.

   This is a simplified version of do_rblik for the fzplod program  
                                                 EAT  Aug 2005           */
{
   double z, zmax;
   int   k;

   zmax = -DBL_MAX;
   for (k = 1; k < ntp; ++k)
   {
/*      if (lleft[k] != lrght[k])                             */
/* Only necessary if trait-marker incompatibility is possible */
      {
/*
 *  printf ("\n starting trait position %d", k); fflush(stdout);
 *  printf ("\n locus left %d recom %f %f,  right %d recom %f %f",
 *  lleft[k], rml[k], rfl[k], lrght[k], rmr[k], rfr[k]);
 *  fflush(stdout);
 */
         z = do_mjrlik (lleft[k], lrght[k], rml[k], rfl[k], rmr[k], rfr[k], 0,
                        NULL);
         /*  Note call is with cmpflag = 0: cmp liks not set               */
         /*  printf ("\n completed peel; like = %e",z); fflush(stdout);    */

         /* checks for Zero probability in peeling operation                  */
         if (z == -DBL_MAX)
            give_up_where (WHERE, ERS_zpp);

         scor[k][t] = z;
         /* for omnibus test we will do only marker posns */
         if ((z > zmax) && (lleft[k] == lrght[k]))
            zmax = z;
      }
   }
   scor[0][t] = zmax;
   return;
}

void do_fzplik2 (int t, int ntp,
                 int *lleft, int *lrght, double *rml, double *rfl,
                 double *rmr, double *rfr, double **scor)
/* ntp = # linked trait positions, at intervals loc and recombination rec,
   loc_left, etc, are recombinations, intervals etc, for trait positions.
   transmissions MUST be computed by set_simple_trans ONLY
   lsample_flag  MUST be 0 -- S's are NOT resampled.

   This is versionfor conf intervals, where currently can NOT do marker
                  posns                          EAT  Oct 2006           */
{
   double z, zmax;
   int   k;

   zmax = -DBL_MAX;
   for (k = 1; k < ntp; ++k)
   {
      if (lleft[k] != lrght[k])
      {
         z = do_mjrlik (lleft[k], lrght[k], rml[k], rfl[k], rmr[k], rfr[k], 0,
                        NULL);
         /*  Note call is with cmpflag = 0: cmp liks not set               */
         /*  printf ("\n completed peel; like = %e",z); fflush(stdout);    */

         /* checks for Zero probability in peeling operation                  */
         if (z == -DBL_MAX)
            give_up_where (WHERE, ERS_zpp);

         scor[k][t] = z;
         /* For conf intervals need overall max  -- not incl markers */
         if (z > zmax)
            zmax = z;
      }
   }
   scor[0][t] = zmax;
   return;
}

void scor_fzplik (double **fzpval, int ntp, int nnull,
                  int *lleft, int *lrght, double *rml, double *rfl,
                  double *rmr, double *rfr, double **qqnull)
{
   int   k;
   double z, zmax;

   zmax = -DBL_MAX;
   for (k = 1; k < ntp; ++k)
   {
      z = do_mjrlik (lleft[k], lrght[k], rml[k], rfl[k], rmr[k], rfr[k], 0,
                     NULL);
      if (z == -DBL_MAX)
         give_up_where (WHERE, ERS_zpp);

      scor_pval (z, nnull, fzpval[k], qqnull[k]);

      if ((z > zmax) && (lleft[k] == lrght[k]))
         zmax = z;
   }
   scor_pval (zmax, nnull, fzpval[0], qqnull[0]);
   return;
}

void scor_fzconf (double **fzpval, int ntp, int nnull,
                  int *lleft, int *lrght, double *rml, double *rfl,
                  double *rmr, double *rfr, double **qqnull)
{
   int   k;
   double z[65], zmax;

   zmax = -DBL_MAX;
   for (k = 1; k < ntp; ++k)
   {
      if (lleft[k] != lrght[k])
      {
         z[k] = do_mjrlik (lleft[k], lrght[k], rml[k], rfl[k], rmr[k],
                           rfr[k], 0, NULL);
         if (z[k] == -DBL_MAX)
            give_up_where (WHERE, ERS_zpp);
         if (z[k] > zmax)
            zmax = z[k];
      }
   }

   for (k = 1; k < ntp; ++k)
   {
      if (lleft[k] != lrght[k])
      {
         z[k] = zmax - z[k];
         scor_pval (z[k], nnull, fzpval[k], qqnull[k]);
      }
   }
   return;
}
