
/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/* do_msmpl(), alloc_msmpl(), free_msmpl()  */

/*******************************************************************
*    M-Sample Step     EAT 1997                                    *
*      to subroutine   EAT March 2000                              *
*    Multiple meiosis version   LPT  2004?                         *
*    Separate out allocs and frees etc   EAT May 2006              *
*    Split the big do_msmpl routine to smaller bits  EAT June 2006 *
*      (so can use forward and back steps separately)              *
********************************************************************/

#include "mulmsample.h"

static int allocated = 0;
static int nm2;
static int **new_segs;                  /* [NLOC][nmeio], NLOC = nl + 1     */
static double **pp2;                    /* [NLOC][2^nmeio]  penetrance probabilities
                                           at each locus under proposed values
                                           for switching given meiosis in given
                                           component  */
static double **pcf;                    /* [NLOC][2^nmeio]  cumulative peeling probs
                                           along chromosome  */

/* alloc and free for msmpl2 */

void alloc_msmpl2 (int nm, int nl)
{
   nm2 = pow2 (nm);

   /*  allocate memories */
   new_segs = alloc_int2d (nl + 1, nm);
   pp2 = alloc_dbl2d (nl + 1, nm2);
   pcf = alloc_dbl2d (nl + 1, nm2);

   allocated = nm;
   return;
}

void free_msmpl2 (int nl)
{
   free_2d ((void *) &new_segs, nl + 1);
   free_2d ((void *) &pp2, nl + 1);
   free_2d ((void *) &pcf, nl + 1);

   allocated = 0;
   return;
}

/* use seg_pen2(), Apr 11, 2006 */
/* c = index of component, c = 1,...,n_compnts 
*/
void do_msmpl2 (int c, int nl, int mm, int *indx, int *sx, int nmeio,
                double **prob, int ng)
/* if mm=1, do not do locus 0 if chromo[0][0]==0 */
{
   int   nm2, mmm;
   double lklhd;                        /* lklhd records the LOD score */

   /* no. of meioses to update must be positive integer */
   if (nmeio <= 0)
      return;

   mmm = mm;
   if (chromo[0][0] != 0)
      mmm = 0;

   nm2 = pow2 (nmeio);

   /*  allocate memories */
   if (nmeio > allocated)
   {
      if (allocated)
         free_msmpl2 (nl);
      alloc_msmpl2 (nmeio, nl);
   }

   /* store all the penetrance values in pp2[NLOC][nm2] */

   store_pen (c, nl, mmm, indx, sx, nmeio, ng, pp2);

   /* compute P(S_j | Y_1, ..., Y_j) by "peeling" */

   lklhd = calc_fwd (nl, mmm, nmeio, sx, pp2, pcf);

   /* sample new indicators by "reverse peeling" */

   bck_smpl (nl, mmm, nmeio, sx, pcf, new_segs);

   /* assign new inheritance indicators to xx0 or xx1 and re-set genes */

   assign_reset (c, nl, mmm, nmeio, indx, sx, ng, prob, new_segs);

   return;
}

#if 0

/* do the same thing as do_msmpl2(), but use seg_pen2_hap, 
   it is more up to date, Apr 11, 2006 */
/* c is the index of component, c = 1,...,n_compnts */
/* Note that do_msmpl_hap () has two more parameters 
   comparing to do_msmpl2 (): *idlist, ni */

void do_msmpl_hap (int c, int nl, int mm, int *idlist, int ni, int *indx,
                   int *sx, int nmeio, double **prob, int ng)
/* if mm=1, do not do locus 0 if chromo[0][0]==0 */
{
   int   i, j, jj, ii, nm2, err, mmm;
   double rr[2];                        /* rr records the recombination frequency 
                                           for female/male */
   double lklhd;                        /* lklhd records the LOD score */

   mmm = mm;
   if (chromo[0][0] != 0)
      mmm = 0;

   nm2 = pow2 (nmeio);

   /*  allocate memories */
   alloc_msmpl2 (nm, nl);

   /* store all the penetrance values in pp2[NLOC][nm2] */

   for (jj = mmm; jj <= nl; jj++)
   {
      for (i = 0; i < nmeio; i++)
         ss[i] = 0;
      for (ii = 0; ii < nm2; ii++)
      {
         for (i = 0; i < nmeio; i++)
         {
            if (sx[i] == 0)
               xx0[jj][indx[i]] = ss[i];
            else
               xx1[jj][indx[i]] = ss[i];
         }
         set_FGL (jj);
         pp2[jj][ii] = seg_pen2_hap (idlist, ni, Qq[jj], jj, ng, &err);
         biadd (ss, nmeio, 1);
      }
   }

   /* compute P(S_j | Y_1, ..., Y_j) by "peeling" */
   if (mmm == 0)
   {
      j = 0;
      jj = chromo[0][j];
      lklhd = fwd_compute (nmeio, j, mmm, NULL, sx, pp2[jj], NULL, pcf[j]);
   }
   for (j = 1; j <= nl; j++)
   {
      jj = chromo[0][j];
      rr[0] = xecomo[0][0][j - 1];
      rr[1] = xecomo[1][0][j - 1];
      lklhd = fwd_compute (nmeio, j, mmm, rr, sx, pp2[jj], pcf[j - 1], pcf[j]);
   }

   /* sample new indicators by "reverse peeling" */
   j = nl;
   rr[0] = xecomo[0][0][j];
   rr[1] = xecomo[1][0][j];
   back_sample2 (nmeio, nl, j, NULL, rr, sx, pcf[j], new_segs[j]);

   for (j = nl - 1; j >= mmm; j--)
   {
      rr[0] = xecomo[0][0][j];
      rr[1] = xecomo[1][0][j];
      back_sample2 (nmeio, nl, j, new_segs[j + 1], rr, sx, pcf[j], new_segs[j]);
   }

   /* assign new inheritance indicators to xx0 or xx1 and re-set genes */

   for (j = mmm; j <= nl; j++)
   {
      jj = chromo[0][j];
      for (i = 0; i < nmeio; i++)
      {
         if (sx[i] == 0)
            xx0[jj][indx[i]] = new_segs[j][i];
         else
            xx1[jj][indx[i]] = new_segs[j][i];
      }
      set_FGL (jj);
      prob[c - 1][jj] = seg_pen2_hap (idlist, ni, Qq[jj], jj, ng, &err);
   }

   free_msmpl2 (nl);

   return;
}
#endif
