/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#include <float.h>
#include "mulmsample.h"

/* Routines to compute the forwards cumulative probabilities
   and then to do the revierse-peeling backwards computation */

/*********************************************************/
/* This function is calculated by Liping in Mar, 2005 */
/* Calculate the fwd variables in restricted sibs updating */

void fwd_compute33 (int meio, int j, int mm, double *r, int *ind,
                    int *sx, double *pen, double *prior, double *new)
/* at locus position j, given penetrances, and the priors to locus 
   position j-1 (prior),
   compute the probs to locus j (new), at recombination freq r  */
{
   int   i, ii, jj1, jj2, t, m2, w1[meio], w2[meio];
   double temp;

   m2 = 4;

   if (j == mm)
      for (i = 0; i < m2; i++)
         new[i] = pen[i];
   else
   {
      jj2 = chromo[0][j];
      jj1 = chromo[0][j - 1];
      for (i = 0; i < m2; i++)
      {
         /* assign values for w2 */

         change_meiosis (0, meio, jj2, ind, sx, i);
         for (t = 0; t < meio; t++)
         {
            if (sx[t] == 0)
               w2[t] = xx0[jj2][ind[t]];
            else
               w2[t] = xx1[jj2][ind[t]];
         }
         change_meiosis (0, meio, jj2, ind, sx, i);

         new[i] = 0;
         for (ii = 0; ii < m2; ii++)
         {
            /* assign values for w1 */

            change_meiosis (0, meio, jj1, ind, sx, ii);
            for (t = 0; t < meio; t++)
            {
               if (sx[t] == 0)
                  w1[t] = xx0[jj1][ind[t]];
               else
                  w1[t] = xx1[jj1][ind[t]];
            }
            change_meiosis (0, meio, jj1, ind, sx, ii);

            new[i] += prior[ii] * probtrans (w2, w1, meio, r, sx);
         }

         new[i] = new[i] * pen[i];
      }
   }

   temp = 0;
   for (i = 0; i < m2; i++)
      temp += new[i];
   for (i = 0; i < m2; i++)
      new[i] = new[i] / temp;

   /*
      printf("\n j=%d, new = %f, %f, %f, %f", j, new[0], new[1], new[2], new[3]);
      getchar();
    */

   return;
}

/*********************************************************/
/* This function is calculated by Liping in Mar, 2005 */
/* Calculate the fwd variables in restricted sibs updating */
/* Rewritten by LPT, Aug, 2006 */

void fwd_compute3 (int meio, int j, int mm, double *r, int *ind,
                   int *sx, double *pen, double *prior, double *new)
/* at locus position j, given penetrances, and the priors to locus 
   position j-1 (prior),
   compute the probs to locus j (new), at recombination freq r  */
{
   int   i, ii, ii1, jj1, jj2, t, m2, tw[2];
   double temp, tr[2][2], *tnew;

   m2 = 4;
   al_double_vector (&tnew, m2);

   if (j == mm)
      for (i = 0; i < m2; i++)
         new[i] = pen[i];
   else
   {
      jj2 = chromo[0][j];
      jj1 = chromo[0][j - 1];
      tr[0][0] = tr[0][1] = tr[1][0] = tr[1][1] = 1.0;

      /* find current values for w1, w2, and tr[][] */
      /* w1 are the meiosis indicators at locus jj1 */
      /* w2 are the meiosis indicators at locus jj2 */
      /* tr[0][0]: similar to r[0]
         tr[0][1]: similar to 1 - r[0]
         tr[1][0]: similar to r[1]
         tr[1][1]: similar to 1 - r[1]
       */

      for (t = 0; t < meio; t++)
      {
         if (sx[t] == 0)
         {
            if (xx0[jj1][ind[t]] == xx0[jj2][ind[t]])
            {
               tr[0][0] *= r[0];
               tr[0][1] *= 1 - r[0];
            }
            else
            {
               tr[0][0] *= 1 - r[0];
               tr[0][1] *= r[0];
            }
         }
         else
         {
            if (xx1[jj1][ind[t]] == xx1[jj2][ind[t]])
            {
               tr[1][0] *= r[1];
               tr[1][1] *= 1 - r[1];
            }
            else
            {
               tr[1][0] *= 1 - r[1];
               tr[1][1] *= r[1];
            }
         }
      }

      /* calculate and normalize new[] */
      for (ii = 0; ii < m2; ii++)
         tnew[ii] = prior[ii];

      for (i = 0; i < 2; i++)
      {
         tw[0] = tw[1] = 0;
         for (ii = 0; ii < m2; ii++)
         {
            if (tw[i] == 0)
               ii1 = ii + pow2 (1 - i);
            else
               ii1 = ii - pow2 (1 - i);
            new[ii] = tnew[ii] * tr[i][1] + tnew[ii1] * tr[i][0];
            biadd (tw, 2, 1);
         }
         for (ii = 0; ii < m2; ii++)
            tnew[ii] = new[ii];
      }

      for (ii = 0; ii < m2; ii++)
         new[ii] = tnew[ii] * pen[ii];
   }

   temp = 0;
   for (i = 0; i < m2; i++)
      temp += new[i];
   for (i = 0; i < m2; i++)
      new[i] = new[i] / temp;

   free_1d ((void *) &tnew);

   return;
}

/*********************************************************/
/* This function is written by Liping in Mar, 2005 */
/* Sample the variables X_j in sibs updating */

int back_sample3 (int meio, int nloc, int j, int right, double *r,
                  int *ind, int *sx, double *left_prob)
/* realise at locus position j, given realisation to the right, at
   recomb freq r, and cumulative penetrance to the left of left_prob 
   for value 0 */
{
   double temp, *cc;
   int   i, k, m2, jj;
   int  *ss1, *ss2;

   m2 = 4;

   al_double_vector (&cc, m2);
   al_int_vector (&ss1, meio);
   al_int_vector (&ss2, meio);

   if (j == nloc)
   {
      for (k = 0; k < m2; k++)
         cc[k] = left_prob[k];
   }
   else
   {
      jj = chromo[0][j + 1];
      change_meiosis (0, meio, jj, ind, sx, right);
      for (i = 0; i < meio; i++)
      {
         if (sx[i] == 0)
            ss2[i] = xx0[jj][ind[i]];
         else
            ss2[i] = xx1[jj][ind[i]];
      }
      change_meiosis (0, meio, jj, ind, sx, right);

      temp = 0;
      jj = chromo[0][j];

      for (k = 0; k < m2; k++)
      {
         change_meiosis (0, meio, jj, ind, sx, k);
         for (i = 0; i < meio; i++)
         {
            if (sx[i] == 0)
               ss1[i] = xx0[jj][ind[i]];
            else
               ss1[i] = xx1[jj][ind[i]];
         }
         change_meiosis (0, meio, jj, ind, sx, k);

         cc[k] = left_prob[k] * probtrans (ss2, ss1, meio, r, sx);
         temp += cc[k];
      }
      for (k = 0; k < m2; k++)
         cc[k] = cc[k] / temp;
   }

   /*
      printf("\n j=%d, cc = ", j);
      for (k = 0; k < m2; k++) printf(" %f,", cc[k]);
      getchar();
    */

   k = 0;
   temp = rans ();
   while (temp > cc[k] && k < m2)
   {
      temp = temp - cc[k];
      k++;
   }

   free_1d ((void *) &cc);
   free_1d ((void *) &ss1);
   free_1d ((void *) &ss2);

   return k;
}

/*********************************************************/
/* This function is calculated by Liping in Mar, 2005 */
/* Calculate the fwd variables in 3-generation updating */

void fwd_compute4 (int ncmeio, int meio, int j, int mm, double *r, int *ind,
                   int *sx, double *pen, double *prior, double *new)
/* at locus position j, given penetrances, and the priors to locus 
   position j-1 (prior),
   compute the probs to locus j (new), at recombination freq r  */
{
   int   i, ii, jj1, jj2, t, m2, w1[meio], w2[meio];
   double temp;

   m2 = 2;
   if (j == mm)
      for (i = 0; i < m2; i++)
         new[i] = pen[i];
   else
   {
      jj2 = chromo[0][j];
      jj1 = chromo[0][j - 1];

      /* currently, the xx0 and xx1 are for change indicator 10 */

      for (i = 0; i < m2; i++)
      {
         /* assign values for w2 */

         for (t = 0; t < meio; t++)
         {
            if (sx[t] == 0)
               w2[t] = xx0[jj2][ind[t]];
            else
               w2[t] = xx1[jj2][ind[t]];
         }
         change_meiosis (ncmeio, meio, jj2, ind, sx, 10);

         new[i] = 0;
         for (ii = 0; ii < m2; ii++)
         {
            /* assign values for w1 */

            for (t = 0; t < meio; t++)
            {
               if (sx[t] == 0)
                  w1[t] = xx0[jj1][ind[t]];
               else
                  w1[t] = xx1[jj1][ind[t]];
            }
            change_meiosis (ncmeio, meio, jj1, ind, sx, 10);

            /* accumulate new[i] */
            new[i] += prior[ii] * probtrans (w2, w1, meio, r, sx);
         }

         new[i] = new[i] * pen[i];
      }
   }

   temp = 0;
   for (i = 0; i < m2; i++)
      temp += new[i];
   for (i = 0; i < m2; i++)
      new[i] = new[i] / temp;

   return;
}

/*********************************************************/
/* This function is written by Liping in Mar, 2005 */
/* Sample the variables X_j in 3-generation updating */

int back_sample4 (int ncmeio, int meio, int nloc, int j, int right, double *r,
                  int *ind, int *sx, double *left_prob)
/* realise at locus position j, given realisation to the right, at
   recomb freq r, and cumulative penetrance to the left of left_prob 
   for value 0 */
{
   double temp, *cc;
   int   i, k, m2, jj;
   int  *ss1, *ss2;

   m2 = 2;

   al_double_vector (&cc, m2);
   al_int_vector (&ss1, meio);
   al_int_vector (&ss2, meio);

   if (j == nloc)
   {
      for (k = 0; k < m2; k++)
         cc[k] = left_prob[k];
   }
   else
   {
      jj = chromo[0][j + 1];

      /* currently, the xx0 and xx1 are for change indicator 10 */

      change_meiosis (ncmeio, meio, jj, ind, sx, 10 - right);
      for (i = 0; i < meio; i++)
      {
         if (sx[i] == 0)
            ss2[i] = xx0[jj][ind[i]];
         else
            ss2[i] = xx1[jj][ind[i]];
      }
      change_meiosis (ncmeio, meio, jj, ind, sx, 10 - right);

      temp = 0;
      jj = chromo[0][j];

      for (k = 0; k < m2; k++)
      {
         change_meiosis (ncmeio, meio, jj, ind, sx, 10);
         for (i = 0; i < meio; i++)
         {
            if (sx[i] == 0)
               ss1[i] = xx0[jj][ind[i]];
            else
               ss1[i] = xx1[jj][ind[i]];
         }

         cc[k] = left_prob[k] * probtrans (ss2, ss1, meio, r, sx);
         temp += cc[k];
      }
      for (k = 0; k < m2; k++)
         cc[k] = cc[k] / temp;
   }

   /*
      printf("\n j=%d, cc = ", j);
      for (k = 0; k < m2; k++) printf(" %f,", cc[k]);
      getchar();
    */

   k = 0;
   temp = rans ();
   if (temp > cc[0])
      k = 10;

   free_1d ((void *) &cc);
   free_1d ((void *) &ss1);
   free_1d ((void *) &ss2);

   return k;
}

/* the following are the forward and backward calculations defined by Liping 
   (different from the above "peeling"). They are used ONLY to verify the 
   calculation of LOD score and to find the exact marginal distribution of 
   meiosis indicators S, that is, Pr(S|Y).
*/

/*********************************************************/
/* This function is first defined by Liping in Nov, 2004 */
/* purpose: calculate log forward variable alpha[m2][nloc]
   input: y[L][M],freq[M],M,L
          r(prob of recombination),q(para related to obs distr)
   output: log of forward variable alpha[M2,L]
*/
void forward2 (int m, int nloc, double **alpha, double **pp2, double *r,
               int *sx)
{
   int   i, j, ii, jj, m2;
   int  *w1, *w2;
   double temp;

   m2 = pow2 (m);

   al_int_vector (&w1, m);
   al_int_vector (&w2, m);

   for (i = 0; i < m; i++)
      w1[i] = 0;

   /* assign initial value */
   for (i = 0; i < m2; i++)
   {
      jj = chromo[0][0];
      r[0] = xecomo[0][0][0];
      r[1] = xecomo[1][0][0];
      alpha[0][i] = log (pp2[jj][i]);   /* log(Pr(Y.1=y[0]|S.1=w1)) */
      alpha[0][i] += -log (m2);         /* Pr(S.1=w1) */
      biadd (w1, m, 1);                 /* change w1 to w1+1 */
   }
   for (j = 1; j <= nloc; j++)
   {
      jj = chromo[0][j];
      r[0] = xecomo[0][0][j - 1];
      r[1] = xecomo[1][0][j - 1];

      for (ii = 0; ii < m; ii++)
         w2[ii] = 0;
      for (i = 0; i < m2; i++)
      {
         alpha[j][i] = -DBL_MAX;
         for (ii = 0; ii < m; ii++)
            w1[ii] = 0;
         for (ii = 0; ii < m2; ii++)
         {
            temp = alpha[j - 1][ii] + log (probtrans (w2, w1, m, r, sx));
            alpha[j][i] = logsum (alpha[j][i], temp);
            biadd (w1, m, 1);
         }
         alpha[j][i] += log (pp2[jj][i]);
         biadd (w2, m, 1);
      }
   }

   free_1d ((void *) &w1);
   free_1d ((void *) &w2);

   return;
}

/*********************************************************/
/* purpose: calculate log backward variable beta[M2,L]
   input: y[M,L],freq[M],M(#meioses),L(#location),
          r(prob of recombination),q(para related to obs distr)
   output: log of backward variable beta[M2,L]
*/
void backward2 (int m, int nloc, double **beta, double **pp2, double *r,
                int *sx)
{
   int   i, j, ii, jj, m2;
   int  *w1, *w2;
   double temp;

   m2 = pow2 (m);

   al_int_vector (&w1, m);
   al_int_vector (&w2, m);

   /* assign initial value */
   for (i = 0; i < m2; i++)
      beta[nloc][i] = 0;

   for (j = nloc - 1; j >= 0; j--)
   {
      jj = chromo[0][j + 1];
      r[0] = xecomo[0][0][j];
      r[1] = xecomo[1][0][j];

      for (ii = 0; ii < m; ii++)
         w2[ii] = 0;
      for (i = 0; i < m2; i++)
      {
         beta[j][i] = -DBL_MAX;
         for (ii = 0; ii < m; ii++)
            w1[ii] = 0;
         for (ii = 0; ii < m2; ii++)
         {
            temp =
               beta[j + 1][ii] + log (probtrans (w2, w1, m, r, sx)) +
               log (pp2[jj][ii]);
            beta[j][i] = logsum (beta[j][i], temp);
            biadd (w1, m, 1);
         }
         biadd (w2, m, 1);
      }
   }

   free_1d ((void *) &w1);
   free_1d ((void *) &w2);

   return;
}
