/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#include "msample.h"
#include "lsample.h"

/****************************************************************************
 *                                                                          *
 *             Simon Heath - MSKCC - June 2001                              *
 *                                                                          *
 * gen_pen.c:                                                               *
 *                                                                          *
 * Calculate p(Y|S) for general penetrance function                         *
 *                                                                          *
 ****************************************************************************/
static struct gp_rfnode **rflist, *free_rfn_list;
static struct gp_rfunc_ptr **rfuncs, *free_rfp_list;
static struct gp_rfunc *free_rf_list[MAX_GP_RF_GENES], **autozyg_list;
static struct deg_list *deglist, **first;
static int *gp_inv, *gp_inv1, *gp_flag, *pflag, *rf_idx, **fact, **fact1;
static int *ofact, *ofact1, rf_idx_n, *rfxn, **rfx, *cnv_gene;
static double *gp_tmp_p, *gp_z;

void alloc_gen_pen (int ngenes, int locus)
{
   int   i, n_all;

   if (CHK_PEEL (TRACE_LEVEL_1))
   {
      printf ("In alloc_gen_pen(%d,%d)\n", ngenes, locus);
      fflush (stdout);
   }
   rflist = malloc (sizeof (void *) * ngenes);
   alloc_chk (WHERE, rflist);
   autozyg_list = malloc (sizeof (void *) * ngenes);
   alloc_chk (WHERE, autozyg_list);
   rfuncs = malloc (sizeof (void *) * ngenes);
   alloc_chk (WHERE, rfuncs);
   first = malloc (sizeof (void *) * ngenes);
   alloc_chk (WHERE, first);
   fact = malloc (sizeof (void *) * 3 * ngenes);
   alloc_chk (WHERE, fact);
   fact1 = fact + ngenes;
   rfx = fact1 + ngenes;
   deglist = malloc (sizeof (struct deg_list) * ngenes);
   alloc_chk (WHERE, deglist);
   for (i = 0; i < ngenes; i++)
      deglist[i].gene = i;
   gp_inv = malloc (sizeof (int) * 8 * ngenes);
   alloc_chk (WHERE, gp_inv);
   gp_inv1 = gp_inv + ngenes;
   gp_flag = gp_inv1 + ngenes;
   pflag = gp_flag + ngenes;
   ofact = pflag + ngenes;
   ofact1 = ofact + ngenes;
   rfxn = ofact1 + ngenes;
   cnv_gene = rfxn + ngenes;
   n_all = loci[locus].n_all;
   gp_tmp_p = malloc (sizeof (double) * (ngenes + n_all * n_all));
   alloc_chk (WHERE, gp_tmp_p);
   gp_z = gp_tmp_p + n_all * n_all;
}

static void free_rfunc (struct gp_rfunc *rf)
{
   if (CHK_PEEL (TRACE_LEVEL_3))
   {
      printf ("In free_rfunc(%p)\n", rf);
      fflush (stdout);
   }
   free (rf->inv);
   free (rf->p);
   free (rf);
}

void free_gen_pen (void)
{
   int   i = 0;
   void **memp[] = { (void **) &rflist, (void **) &deglist, (void **) &first,
      (void **) &gp_inv, (void **) &rfuncs, (void **) &gp_tmp_p,
      (void **) &rf_idx, (void **) &fact, (void **) &autozyg_list, 0
   }, **p;
   struct gp_rfunc_ptr *rfp, *rfp1;
   struct gp_rfnode *rfn, *rfn1;
   struct gp_rfunc *rf, *rf1;

   if (CHK_PEEL (TRACE_LEVEL_1))
   {
      printf ("In free_gen_pen()\n");
      fflush (stdout);
   }
   if (!rflist)
      return;

   rfp = free_rfp_list;
   while (rfp)
   {
      rfp1 = rfp->next;
      free (rfp);
      rfp = rfp1;
   }
   free_rfp_list = 0;
   rfn = free_rfn_list;
   while (rfn)
   {
      rfn1 = rfn->next;
      free (rfn);
      rfn = rfn1;
   }
   free_rfn_list = 0;
   while (memp[i])
   {
      p = memp[i++];
      if (*p)
      {
         free (*p);
         *p = 0;
      }
   }
   rf_idx_n = 0;
   for (i = 0; i < MAX_GP_RF_GENES; i++)
   {
      rf = free_rf_list[i];
      while (rf)
      {
         rf1 = rf->next;
         free_rfunc (rf);
         rf = rf1;
      }
      free_rf_list[i] = 0;
   }
   free_penmat ();
   return;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "get_new_rfunc"
static struct gp_rfunc *get_new_rfunc (int n, int n_all)
{
   int   nc;
   struct gp_rfunc *rf;

   if (n > MAX_GP_RF_GENES || n < 1)
      ABT_FUNC ("Illegal R-Function size\n");
   rf = free_rf_list[n - 1];
   if (rf)
   {
      free_rf_list[n - 1] = rf->next;
      if (CHK_PEEL (TRACE_LEVEL_3))
      {
         printf ("In get_new_rfunc(%d,%d): getting rfunc %p from free list\n",
                 n, n_all, rf);
         fflush (stdout);
      }
   }
   else
   {
      rf = malloc (sizeof (struct gp_rfunc));
      alloc_chk (WHERE, rf);
      rf->n_inv = n;
      nc = (int) (.5 + exp (log ((double) n_all) * (double) n));
      rf->nc = nc;
      rf->p = malloc (sizeof (double) * nc);
      alloc_chk (WHERE, rf->p);
      rf->inv = malloc (sizeof (int) * n);
      alloc_chk (WHERE, rf->inv);
      if (CHK_PEEL (TRACE_LEVEL_3))
         printf ("In get_new_rfunc(%d,%d): allocating new rfunc %p\n", n, n_all,
                 rf);
   }
   rf->next = 0;
   rf->flag = 0;
   return rf;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "get_new_rfptr"
static struct gp_rfunc_ptr *get_new_rfptr (void)
{
   struct gp_rfunc_ptr *rfp;

   if (free_rfp_list)
   {
      rfp = free_rfp_list;
      if (CHK_PEEL (TRACE_LEVEL_3))
         printf ("In get_new_rfptr(): getting rfptr (%p) from free list\n",
                 rfp);
      free_rfp_list = rfp->next;
   }
   else
   {
      rfp = malloc (sizeof (struct gp_rfunc_ptr));
      alloc_chk (WHERE, rfp);
      if (CHK_PEEL (TRACE_LEVEL_3))
         printf ("In get_new_rfptr(): allocating new rfptr (%p)\n", rfp);
   }
   rfp->next = 0;
   rfp->rf = 0;
   return rfp;
}

static struct gp_rfnode *get_new_rfnode (int y)
{
   struct gp_rfnode *p;

   if (free_rfn_list)
   {
      p = free_rfn_list;
      free_rfn_list = p->next;
      if (CHK_PEEL (TRACE_LEVEL_3))
         printf ("In get_new_rfnode(%d): getting rfn (%p) from free list\n", y,
                 p);
   }
   else
   {
      p = malloc (sizeof (struct gp_rfnode));
      alloc_chk (WHERE, p);
      if (CHK_PEEL (TRACE_LEVEL_3))
         printf ("In get_new_rfnode(%d): allocating new rfn (%p)\n", y, p);
   }
   p->next = 0;
   p->rf = 0;
   p->y = y;
   return p;
}

static struct gp_rfnode *find_rfnode (int x, int y)
{
   struct gp_rfnode *p, **pp;

   pp = rflist + x;
   p = *pp;
   while (p)
   {
      if (p->y <= y)
         break;
      pp = &p->next;
      p = *pp;
   }
   if (!p || p->y != y)
   {
      p = get_new_rfnode (y);
      p->next = *pp;
      *pp = p;
   }
   return p;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "delete_rfnode"
static void delete_rfnode (int x, int y)
{
   struct gp_rfnode *p, **pp;

   pp = rflist + x;
   p = *pp;
   while (p)
   {
      if (p->y <= y)
         break;
      pp = &p->next;
      p = *pp;
   }
   if (!p || p->y != y)
   {
      fprintf (stderr, "node %d,%d not found", x, y);
      ABT_FUNC ("aborting\n");
   }
   *pp = p->next;
   p->next = free_rfn_list;
   free_rfn_list = p;
}

static int get_inv_list (int *inv, int i)
{
   int   k = 1;
   struct gp_rfnode *rfn;

   inv[0] = i;
   rfn = rflist[i];
   while (rfn)
   {
      inv[k++] = rfn->y;
      rfn = rfn->next;
   }
   return k;
}

static void del_rfrow (int i)
{
   struct gp_rfnode *rfn;

   rfn = rflist[i];
   if (rfn)
   {
      while (rfn->next)
      {
         delete_rfnode (rfn->y, i);
         rfn = rfn->next;
      }
      delete_rfnode (rfn->y, i);
      rfn->next = free_rfn_list;
      free_rfn_list = rflist[i];
      rflist[i] = 0;
      if (CHK_PEEL (TRACE_LEVEL_3))
         printf ("In del_rfrow(): Adding row %d to free_rfn_list\n", i);
   }
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "calc_degree"
static int calc_degree (int i)
{
   int   i1, j, j1, j2, k = 0, k1, k2;
   struct deg_list *p;
   struct gp_rfnode *rfn;

   if (rflist[i])
   {
      k2 = k = get_inv_list (gp_inv, i);
      gp_inv[k] = -1;
      for (j = 1; j < k; j++)
      {
         i1 = gp_inv[j];
         if (!gp_flag[i1])
         {
            rfn = rflist[i1];
            if (rfn->next)
            {
               j2 = 1;
               while (rfn)
               {
                  k1 = rfn->y;
                  if (k1 != i)
                  {
                     if (gp_inv[j2] == i1)
                        j2++;
                     if (gp_inv[j2++] != k1)
                        break;
                     rfn = rfn->next;
                  }
                  else
                  {
                     rfn = rfn->next;
                     if (!rfn && gp_inv[j2] == i1)
                        j2++;
                  }
               }
               if (!rfn && j2 == k - 1 && gp_inv[j2] == i1)
                  k1 = 1;
               else
                  k1 = (!rfn && j2 == k) ? 1 : 0;
            }
            else
            {
               k1 = k == 2 ? 1 : 0;
            }
            if (k1)
            {
               if (CHK_PEEL (TRACE_LEVEL_2))
                  printf ("Absorbing gene %d into %d\n", i1, i);
               gp_flag[i1] |= 1;
               p = deglist[i1].abs_list;
               if (p)
               {
                  while (p->abs_list)
                     p = p->abs_list;
                  p->abs_list = deglist[i].abs_list;
               }
               else
                  deglist[i1].abs_list = deglist[i].abs_list;
               deglist[i].abs_list = deglist + i1;
               if (deglist[i1].deg >= 0)
               {
                  j1 = deglist[i1].deg;
                  if (CHK_PEEL (TRACE_LEVEL_2))
                     printf ("Removing %d from degree list %d (b)\n", i1, j1);
                  if (deglist[i1].prev)
                     deglist[i1].prev->next = deglist[i1].next;
                  else
                     first[j1] = deglist[i1].next;
                  if (deglist[i1].next)
                     deglist[i1].next->prev = deglist[i1].prev;
                  deglist[i1].deg = -1;
               }
               rfn = rflist[i1];
               while (rfn)
               {
                  k1 = rfn->y;
                  if (k1 != i && !gp_flag[k1])
                  {
                     j2 = deglist[k1].deg;
                     if (j2 >= 0)
                     {
                        if (CHK_PEEL (TRACE_LEVEL_2))
                           printf ("Moving %d from degree list %d to %d (b)\n",
                                   k1, j2, j2 - 1);
                        if (!j2)
                           ABT_FUNC ("Internal error - degree shouldn't be zero"
                                     "\n");
                        if (deglist[k1].prev)
                           deglist[k1].prev->next = deglist[k1].next;
                        else
                           first[j2] = deglist[k1].next;
                        if (deglist[k1].next)
                           deglist[k1].next->prev = deglist[k1].prev;
                        deglist[k1].next = first[--j2];
                        deglist[k1].prev = 0;
                        if (first[j2])
                           first[j2]->prev = deglist + k1;
                        first[j2] = deglist + k1;
                        deglist[k1].deg--;
                     }
                  }
                  rfn = rfn->next;
               }
            }
         }
      }
   }
   else if (autozyg_list[i])
      k = 1;
   return k - 1;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "clean_up"
/* Only called if we exit with an incompatibility */
static void clean_up (int ngenes, struct gp_rfunc *rfl)
{
   int   i;
   struct gp_rfunc_ptr *rfp1, *rfp2;
   struct gp_rfunc *rf;

   for (i = 0; i < ngenes; i++)
   {
      rfp1 = rfuncs[i];
      while (rfp1)
      {
         rfp2 = rfp1->next;
         rf = rfp1->rf;
         if (!rf->flag)
         {
            rf->flag = 1;
            rf->next = rfl;
            rfl = rf;
         }
         if (CHK_PEEL (TRACE_LEVEL_3))
            printf ("[%s:%d] %s() - Adding %p to free_rfp_list\n", __FILE__,
                    __LINE__, FUNC_NAME, rfp1);
         rfp1->next = free_rfp_list;
         free_rfp_list = rfp1;
         rfp1 = rfp2;
      }
      del_rfrow (i);
   }
   while (rfl)
   {
      rf = rfl->next;
      i = rfl->n_inv - 1;
      if (CHK_PEEL (TRACE_LEVEL_3))
         printf ("[%s:%d] %s() - Adding %p to free_rf_list[%d]\n", __FILE__,
                 __LINE__, FUNC_NAME, rfl, i);
      rfl->next = free_rf_list[i];
      free_rf_list[i] = rfl;
      rfl = rf;
   }
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "gen_pen"
double gen_pen (int c, int *genes[2], double *freq, int locus,
                int ngenes, int *err)
{
   int   i, i1, i2, j, k, k1, k2, a1, a2, n_all, n_peel, n_inv, n_out, n_all2,
      *inv, nc = 0;
   int   n_rf, *ff, *idx, out_idx;
   Ind  *cpind;
   Indlist *iptr;
   double lam = 1.0, *xx = 0, like = 0.0, z1, z, z3;
   struct gp_rfnode *rfnode;
   struct gp_rfunc *rf, *rf1, *rfl;
   struct gp_rfunc_ptr *rfp, *rfp1, *rfp2, **rfp3;
   struct deg_list *degp, *degp1;
   struct Locus *loc;

   loc = loci + locus;
   if (CHK_PEEL (TRACE_LEVEL_1))
   {
      printf ("\nIn gen_pen(%d,%p,%p,%d,%d,%p)\n",
              c, genes, freq, locus, ngenes, err);
      fflush (stdout);
   }
   if (!loc->penetrance)
   {
      *err = -1;
      if (CHK_PEEL (TRACE_LEVEL_1))
         fputs ("Returning err=-1, p = 0.0 (no penetrance function)\n", stdout);
      return 0.0;
   }
   n_all = loci[locus].n_all;
   n_all2 = n_all * n_all;
   *err = 0;
   /* Build up initial (penetrance) R-Functions */
   for (i = 0; i < ngenes; i++)
   {
      rflist[i] = 0;
      rfuncs[i] = 0;
      autozyg_list[i] = 0;
   }
   iptr = cmplist[c + 1].people;
   while (iptr->next != NULL)
   {
      iptr = iptr->next;
      cpind = iptr->current;
      i = cpind->index;
      if (nghd[i].obs)
      {
         a1 = genes[0][i] - 1;
         a2 = genes[1][i] - 1;
         if (CHK_PEEL (TRACE_LEVEL_2))
         {
            printf ("Ind %s: genes %d %d\n", cpind->self, a1, a2);
            fflush (stdout);
         }
         if (a1 == a2)
         {
            rf = autozyg_list[a1];
            if (!rf)
            {
               /* Allocate rfunction when a1==a2 */
               rf = get_new_rfunc (1, n_all);
               rf->inv[0] = a1;
               rfp = get_new_rfptr ();
               rfp->next = rfuncs[a1];
               rfp->rf = rf;
               rfuncs[a1] = rfp;
               xx = rf->p;
               for (j = 0; j < n_all; j++)
                  xx[j] = 1.0;
               autozyg_list[a1] = rf;
            }
         }
         else
         {
            if (a1 > a2)
            {
               k = a1;
               a1 = a2;
               a2 = k;
            }
            rfnode = find_rfnode (a1, a2);
            if (!rfnode->rf)
            {                           /* Is this a new combination? */
               /* Allocate rfunction */
               rf = get_new_rfunc (2, n_all);
               rf->inv[0] = a1;
               rf->inv[1] = a2;
               rfnode->rf = rf;
               rfp = get_new_rfptr ();
               rfp->next = rfuncs[a1];
               rfp->rf = rf;
               rfuncs[a1] = rfp;
               /* Add in reference to symmetrical node */
               rfp = get_new_rfptr ();
               rfp->next = rfuncs[a2];
               rfp->rf = rf;
               rfuncs[a2] = rfp;
               rfnode = find_rfnode (a2, a1);
               rfnode->rf = rf;
               xx = rf->p;
               for (j = 0; j < n_all2; j++)
                  xx[j] = 1.0;
            }
            else
               rf = rfnode->rf;
         }
         xx = rf->p;
         /* Add in penetrance */
         if (a1 != a2)
         {
            loc->penetrance (xx, cpind, loc, lam);
            if (CHK_PEEL (TRACE_LEVEL_4))
               for (k1 = j = 0; j < n_all; j++)
               {
                  for (k = 0; k < n_all; k++)
                  {
                     printf ("%d %d : %g\n", j, k, xx[k1++]);
                  }
               }
         }
         else
         {
            for (j = 0; j < n_all2; j++)
               gp_tmp_p[j] = 0.0;
            for (j = 0; j < n_all; j++)
               gp_tmp_p[n_all * j + j] = 1.0;
            loc->penetrance (gp_tmp_p, cpind, loc, lam);
            for (j = 0; j < n_all; j++)
               xx[j] = gp_tmp_p[n_all * j + j];
            if (CHK_PEEL (TRACE_LEVEL_4))
               for (j = 0; j < n_all; j++)
                  printf ("%d : %g\n", j, xx[j]);
         }
      }
   }
   if (CHK_PEEL (TRACE_LEVEL_3))
   {
      fputs ("Node adjacancies\n", stdout);
      for (i = 0; i < ngenes; i++)
      {
         rfnode = rflist[i];
         if (rfnode)
         {
            printf ("%d:", i);
            while (rfnode)
            {
               printf (" %d", rfnode->y);
               rfnode = rfnode->next;
            }
            (void) fputc ('\n', stdout);
         }
      }
   }
   /* Find peeling sequence using the minimum external degree approach */
   /* Find degree of each node and put into linked list */
   for (i = 0; i < ngenes; i++)
   {
      first[i] = 0;
      gp_flag[i] = 0;
      deglist[i].abs_list = 0;
      deglist[i].deg = -1;
   }
   for (i = 0; i < ngenes; i++)
   {
      if (!gp_flag[i])
      {
         k = calc_degree (i);
         if (k >= 0)
         {
            j = 0;
            for (k1 = 1; k1 <= k; k1++)
               if (!gp_flag[gp_inv[k1]])
                  j++;
            deglist[i].deg = j;
            deglist[i].next = first[j];
            deglist[i].prev = 0;
            if (first[j])
               first[j]->prev = deglist + i;
            first[j] = deglist + i;
         }
      }
   }
   if (CHK_PEEL (TRACE_LEVEL_3))
   {
      fputs ("Degree lists\n", stdout);
      for (i = 0; i < ngenes; i++)
      {
         degp = first[i];
         if (degp)
         {
            printf ("%d:", i);
            while (degp)
            {
               j = degp->gene;
               printf (" %d", j);
               degp1 = degp->abs_list;
               while (degp1)
               {
                  printf ("-%d", degp1->gene);
                  degp1 = degp1->abs_list;
               }
               degp = degp->next;
            }
            (void) fputc ('\n', stdout);
         }
      }
   }
   /* Main peeling loop */
   for (;;)
   {
      /* Pick a node of minimum degree */
      for (i = 0; i < ngenes; i++)
         if (first[i])
            break;
      if (i == ngenes)
         break;
      j = first[i]->gene;
      if (gp_flag[j])
      {
         fprintf (stderr, "Internal error when peeling gene %d from degree list"
                  " %d\n", j, i);
         ABT_FUNC ("Gene already peeled\n");
      }
      /* Get list of genes involved */
      n_inv = get_inv_list (gp_inv, j);
      /* Find out who we can peel */
      n_peel = 1;
      gp_inv1[0] = j;
      gp_flag[j] |= 4;
      degp = deglist[j].abs_list;
      while (degp)
      {
         j = degp->gene;
         gp_flag[j] |= 2;
         gp_inv1[n_peel++] = j;
         degp = degp->abs_list;
      }
      k = n_peel;
      for (i = 1; i < n_inv; i++)
      {
         j = gp_inv[i];
         if (!(gp_flag[j] & 2))
         {
            gp_inv1[k++] = j;
         }
      }
      if (n_inv > 1)
         qsort (gp_inv1, n_inv, sizeof (int), qfunc_int);
      for (i = 0; i < n_inv; i++)
         cnv_gene[gp_inv1[i]] = i;
      if (CHK_PEEL (TRACE_LEVEL_2))
      {
         fputs ("Peeling", stdout);
         for (i = j = 0; i < n_inv; i++)
         {
            k = gp_inv1[i];
            if (gp_flag[k] & 6)
            {
               (void) fputc (j++ ? ',' : ' ', stdout);
               printf ("%d", k);
            }
         }
         fputs (" ->", stdout);
         if (n_peel == n_inv)
            fputs (" *", stdout);
         else
         {
            for (i = j = 0; i < n_inv; i++)
            {
               k = gp_inv1[i];
               if (!(gp_flag[k] & 6))
               {
                  (void) fputc (j++ ? ',' : ' ', stdout);
                  printf ("%d", k);
               }
            }
         }
      }
      /* Get list of rfunctions for nodes to be peeled */
      for (rfl = 0, n_rf = i = 0; i < n_inv; i++)
      {
         k = gp_inv1[i];
         if (gp_flag[k] & 6)
         {
            pflag[i] = 1;
            rfp1 = rfuncs[k];
            while (rfp1)
            {
               rfp2 = rfp1->next;
               rf1 = rfp1->rf;
               if (!rf1->flag)
               {
                  rf1->next = rfl;
                  rfl = rf1;
                  rf1->flag = 1;
                  n_rf++;
               }
               if (CHK_PEEL (TRACE_LEVEL_3))
                  printf ("\n[%s:%d] %s() - Adding %p to free_rfp_list\n",
                          __FILE__, __LINE__, FUNC_NAME, rfp1);
               rfp1->next = free_rfp_list;
               free_rfp_list = rfp1;
               rfp1 = rfp2;
            }
            rfuncs[k] = 0;
         }
         else
            pflag[i] = 0;
      }
      if (CHK_PEEL (TRACE_LEVEL_2))
      {
         rf1 = rfl;
         while (rf1)
         {
            (void) fputc (' ', stdout);
            for (i = 0; i < rf1->n_inv; i++)
            {
               (void) fputc (i ? ',' : '(', stdout);
               printf ("%d", rf1->inv[i]);
            }
            (void) fputc (')', stdout);
            rf1 = rf1->next;
         }
         (void) fputc ('\n', stdout);
      }
      /* Convert genes in rfunctions to references in gp_inv1 */
      rf1 = rfl;
      while (rf1)
      {
         for (i = 0; i < rf1->n_inv; i++)
         {
            j = rf1->inv[i];
            rf1->inv[i] = cnv_gene[j];
         }
         rf1 = rf1->next;
      }
      /* Setup output rfunction */
      n_out = n_inv - n_peel;
      if (n_out)
      {
         rf = get_new_rfunc (n_out, n_all);
         xx = rf->p;
         nc = rf->nc;
         for (i = 0; i < nc; i++)
            xx[i] = 0.0;
         for (j = 0, i = 0; i < n_inv; i++)
         {
            if (!pflag[i])
            {
               k1 = gp_inv1[i];
               rf->inv[j++] = k1;
               rfp1 = get_new_rfptr ();
               rfp1->next = rfuncs[k1];
               rfp1->rf = rf;
               rfuncs[k1] = rfp1;
               /* Fill in new adjacencies */
               for (i1 = 0; i1 < i; i1++)
               {
                  if (!pflag[i1])
                  {
                     k2 = gp_inv1[i1];
                     rfnode = find_rfnode (k1, k2);
                     rfnode = find_rfnode (k2, k1);
                  }
               }
            }
         }
      }
      z1 = 0.0;
      /* Perform peelop */
      /* Set allele indices */
      for (i = 0; i < n_inv; i++)
         gp_inv[i] = 0;
      z = 1.0;
      for (i = n_inv - 1; i >= 0; i--)
      {
         gp_z[i] = z;
         if (pflag[i])
            z *= freq[0];
      }
      z3 = z;
      /* Initialize rfunction indices */
      k = 0;
      rf1 = rfl;
      i1 = n_all - 1;
      if (n_rf > rf_idx_n)
      {
         rf_idx_n = n_rf;
         if (rf_idx)
            rf_idx = realloc (rf_idx, sizeof (int) * (rf_idx_n + 3 * rf_idx_n
                                                      * ngenes));
         else
            rf_idx = malloc (sizeof (int) * (rf_idx_n + 3 * rf_idx_n * ngenes));
         alloc_chk (WHERE, rf_idx);
         fact[0] = rf_idx + rf_idx_n;
         fact1[0] = fact[0] + rf_idx_n * ngenes;
         rfx[0] = fact1[0] + rf_idx_n * ngenes;
         for (k1 = 1; k1 < ngenes; k1++)
         {
            fact[k1] = fact[k1 - 1] + rf_idx_n;
            fact1[k1] = fact1[k1 - 1] + rf_idx_n;
            rfx[k1] = rfx[k1 - 1] + rf_idx_n;
         }
      }
      for (k1 = 0; k1 < n_inv; k1++)
         rfxn[k1] = 0;
      while (rf1)
      {
         inv = rf1->inv;
         for (k2 = 1, k1 = 0; k1 < rf1->n_inv; k1++, k2 *= n_all)
         {
            i2 = inv[k1];
            fact[i2][k] = k2;
            fact1[i2][k] = k2 * i1;
            rfx[i2][rfxn[i2]++] = k;
         }
         rf_idx[k++] = 0;
         rf1 = rf1->next;
      }
      /* Initialize output index */
      for (k2 = 1, k1 = 0; k1 < n_inv; k1++)
      {
         if (pflag[k1])
            ofact[k1] = ofact1[k1] = 0;
         else
         {
            ofact[k1] = k2;
            ofact1[k1] = k2 * i1;
            k2 *= n_all;
         }
      }
      out_idx = 0;
      do
      {                                 /* Loop through all combinations */
         if (CHK_PEEL (TRACE_LEVEL_4))
            for (j = 0; j < n_inv; j++)
               printf ("%d ", gp_inv[j]);
         /* Insert frequencies for peeled genes */
         z = z3;
         /* Insert probabilities from rfunctions */
         idx = rf_idx;
         rf1 = rfl;
         while (rf1)
         {
            z *= rf1->p[*(idx++)];
            rf1 = rf1->next;
         }
         if (n_out)
            xx[out_idx] += z;
         z1 += z;
         if (CHK_PEEL (TRACE_LEVEL_4))
         {
            k1 = 0;
            rf1 = rfl;
            while (rf1)
            {
               (void) fputc (k1 ? ',' : '(', stdout);
               printf ("[%d,%g]", rf_idx[k1], rf1->p[rf_idx[k1]]);
               k1++;
               rf1 = rf1->next;
            }
            if (k1)
               fputs (") ", stdout);
            printf (": %g %g\n", z3, z);
         }
         /* Bump indices to get next combination */
         for (i = 0; i < n_inv; i++)
         {
            gp_inv[i]++;
            if (gp_inv[i] < n_all)
            {
               ff = fact[i];
               k1 = rfxn[i];
               inv = rfx[i];
               for (k = 0; k < k1; k++)
               {
                  k2 = inv[k];
                  rf_idx[k2] += ff[k2];
               }
               out_idx += ofact[i];
               break;
            }
            gp_inv[i] = 0;
            ff = fact1[i];
            k1 = rfxn[i];
            inv = rfx[i];
            for (k = 0; k < k1; k++)
            {
               k2 = inv[k];
               rf_idx[k2] -= ff[k2];
            }
            out_idx -= ofact1[i];
         }
         /* Update frequency contribution from peeled genes */
         if (i < n_inv)
         {
            z3 = gp_z[i];
            if (pflag[i])
               z3 *= freq[gp_inv[i]];
            for (k = i - 1; k >= 0; k--)
            {
               gp_z[k] = z3;
               if (pflag[k])
               {
                  z3 *= freq[gp_inv[k]];
               }
            }
         }
      }
      while (i < n_inv);
      if (z1 <= 0.0)
      {
         if (CHK_PEEL (TRACE_LEVEL_1))
            fputs ("Returning err=-1, p = 0.0 (zero prob.) (c)\n", stdout);
         clean_up (ngenes, rfl);
         *err = -1;
         return 0.0;
      }
      /* Rescale output rfunction */
      if (n_out)
      {
         z = 1.0 / z1;
         for (i = 0; i < nc; i++)
            xx[i] *= z;
      }
      like += log (z1);
      if (CHK_PEEL (TRACE_LEVEL_3))
         printf ("z1 = %g\n", z1);
      /* Remove references to used rfunctions from *rfuncs */
      for (i = 0; i < n_inv; i++)
         if (!pflag[i])
         {
            k = gp_inv1[i];
            rfp3 = rfuncs + k;
            rfp1 = *rfp3;
            while (rfp1)
            {
               rfp2 = rfp1->next;
               if (rfp1->rf->flag)
               {
                  if (CHK_PEEL (TRACE_LEVEL_3))
                     printf ("[%s:%d] %s() - Adding %p to free_rfp_list\n",
                             __FILE__, __LINE__, FUNC_NAME, rfp1);
                  *rfp3 = rfp1->next;
                  rfp1->next = free_rfp_list;
                  free_rfp_list = rfp1;
               }
               else
                  rfp3 = &rfp1->next;
               rfp1 = rfp2;
            }
         }
      /* Free used rfunctions */
      rf1 = rfl;
      while (rf1)
      {
         rf = rf1->next;
         k = rf1->n_inv - 1;
         if (CHK_PEEL (TRACE_LEVEL_3))
            printf ("[%s:%d] %s() - Adding %p to free_rf_list[%d]\n", __FILE__,
                    __LINE__, FUNC_NAME, rf1, k);
         rf1->next = free_rf_list[k];
         free_rf_list[k] = rf1;
         rf1 = rf;
      }
      /* Remove peeled nodes from degree list */
      for (i = 0; i < n_inv; i++)
         if (pflag[i])
         {
            k = gp_inv1[i];
            if (!(gp_flag[k] & 2))
            {
               if (CHK_PEEL (TRACE_LEVEL_2))
                  printf ("Removing %d from degree list %d (a)\n", k,
                          deglist[k].deg);
               if (deglist[k].prev)
                  deglist[k].prev->next = deglist[k].next;
               else
               {
                  k1 = deglist[k].deg;
                  first[k1] = deglist[k].next;
               }
               if (deglist[k].next)
                  deglist[k].next->prev = deglist[k].prev;
            }
            del_rfrow (k);
            /* Mark gene as peeled */
            gp_flag[k] |= 2;
         }
      /* Recalculate degree for output nodes */
      for (i = 0; i < n_inv; i++)
         if (!pflag[i])
         {
            k = gp_inv1[i];
            if (!gp_flag[k])
            {
               k1 = calc_degree (k);
               if (k1 >= 0)
               {
                  k2 = 0;
                  for (j = 1; j <= k1; j++)
                     if (!gp_flag[gp_inv[j]])
                        k2++;
               }
               else
                  k2 = k1;
               k1 = deglist[k].deg;
               if (k1 != k2)
               {
                  if (CHK_PEEL (TRACE_LEVEL_2))
                     printf ("Moving %d from degree list %d to %d (a)\n", k, k1,
                             k2);
                  if (deglist[k].prev)
                     deglist[k].prev->next = deglist[k].next;
                  else
                     first[k1] = deglist[k].next;
                  if (deglist[k].next)
                     deglist[k].next->prev = deglist[k].prev;
                  deglist[k].next = first[k2];
                  deglist[k].prev = 0;
                  if (first[k2])
                     first[k2]->prev = deglist + k;
                  first[k2] = deglist + k;
                  deglist[k].deg = k2;
               }
            }
         }
   }
   if (CHK_PEEL (TRACE_LEVEL_1))
   {
      printf ("Returning like = %g, p = %g\n", like, exp (like));
      fflush (stdout);
   }
   return exp (like);
}
