/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#include "lsample.h"
#include "msample.h"
#include "mulmsample.h"

/* update a random number of meioses each time, conditional on 
   all the other meiosis indicators */
/* c = 1,...,n_compnts
*/
void update_meio (int c, int *idlist, int nid, int *swlist, int nsw,
                  int *mrlist, int nmr, int nloc, int mm, double **prob,
                  int ngenes)
{
   int   tt, k, l;
   int  *sx, *ind, nupdate;

   permute (swlist, nsw);               /* permute switches for M-sampler */

   k = 0;
   while (k < nsw)
   {
      /* update some number of meiosis */
      tt = intran (1, Max_update);
      nupdate = min (tt, (nsw - k));

      al_int_vector (&ind, nupdate);
      al_int_vector (&sx, nupdate);

      for (tt = 0; tt < nupdate; tt++)
      {
         l = swlist[k++];
         ind[tt] = (meiosis[l].ind).index;
         sx[tt] = meiosis[l].sex;
      }

      do_msmpl2 (c, nloc, mm, ind, sx, nupdate, prob, ngenes);

      free_1d ((void *) &ind);
      free_1d ((void *) &sx);

   }
   return;
}

/* update meioses by individuals in component c, c start from 1 */

void update_ind (int c, int *idlist, int nid, int *swlist, int nsw, int *mrlist,
                 int nmr, int nloc, int mm, double **prob, int ngenes)
{
   int   i, ii, l;
   int  *sx, *ind, nupdate;

   permute (idlist, nid);               /* permute individuals for M-sampler */

   for (ii = 0; ii < nid; ii++)
   {
      /* the no. of meioses for joint updating is 2 at most */
      al_int_vector (&ind, 2);
      al_int_vector (&sx, 2);

      nupdate = 0;
      i = idlist[ii];
      l = nghd[i].matmeio;              /* l is the index of i's maternal meiosis */
      if (l > 0)
      {
         ind[nupdate] = (meiosis[l].ind).index;
         sx[nupdate] = meiosis[l].sex;
         nupdate++;
      }
      l = nghd[i].patmeio;
      if (l > 0)
      {
         ind[nupdate] = (meiosis[l].ind).index;
         sx[nupdate] = meiosis[l].sex;
         nupdate++;
      }

      do_msmpl2 (c, nloc, mm, ind, sx, nupdate, prob, ngenes);

      free_1d ((void *) &ind);
      free_1d ((void *) &sx);
   }
   return;
}

/* update meioses by sibs in component c, c start from 1 */
/* for each component, consider all the loci jointly */

void update_sib (int c, int *idlist, int nid, int *swlist, int nsw, int *mrlist,
                 int nmr, int nloc, int mm, double **prob, int ngenes)
{
   int   i, j, tt, jj, k, l, t, mmm, err;
   int  *sx, *ind, nupdate;

   double **pp2, **pcf, rr[2];
   int  *xx;

   pp2 = alloc_dbl2d (nloc + 1, 4);
   pcf = alloc_dbl2d (nloc + 1, 4);
   al_int_vector (&xx, nloc + 1);

   mmm = mm;
   if (chromo[0][0] != 0)
      mmm = 0;
   /*
      for (k = 0; k < 4; k++)
      pcf[0][k] = 0;
    */

   permute (mrlist, nmr);               /* permute marriages for M-sampler */

   for (tt = 0; tt < nmr; tt++)
   {
      i = mrlist[tt];
      nupdate = marr[i].nkidmeio;

      /* consider to update meioses of sibs */
      if (nupdate > 0)
      {
         al_int_vector (&ind, nupdate);
         al_int_vector (&sx, nupdate);
         for (t = 0; t < nupdate; t++)
         {
            l = marr[i].kidmeio[t];
            ind[t] = (meiosis[l].ind).index;
            sx[t] = meiosis[l].sex;
         }

         if (nupdate <= Max_update)     /* *** full sib update *** */
         {
            do_msmpl2 (c, nloc, mm, ind, sx, nupdate, prob, ngenes);
         }
         else                           /* *** restricted sib update *** */
         {
            /* store all the penetrance values in pp2[NLOC][4] */
            /* 4 updating type, given current realization of meisis indicators */

            for (j = mmm; j <= nloc; j++)
            {
               pp2[j][0] = prob[c - 1][j];

               for (k = 1; k < 4; k++)
               {
                  change_meiosis (0, nupdate, j, ind, sx, k);
                  set_FGL (j);
                  pp2[j][k] = seg_pen2 (c - 1, loci[j].freq, j, ngenes, &err);

                  /* do it again to restore original xx0 and xx1 */
                  change_meiosis (0, nupdate, j, ind, sx, k);
               }
            }

            /* compute P(X_j | Y_1, ..., Y_j, S_old) by "peeling" */

            if (mmm == 0)
            {
               j = 0;
               jj = chromo[0][j];
               fwd_compute3 (nupdate, j, mm, NULL, ind, sx, pp2[jj], NULL,
                             pcf[j]);
            }
            for (j = 1; j <= nloc; j++)
            {
               jj = chromo[0][j];
               rr[0] = xecomo[0][0][j - 1];
               rr[1] = xecomo[1][0][j - 1];
               fwd_compute3 (nupdate, j, mm, rr, ind, sx, pp2[jj], pcf[j - 1],
                             pcf[j]);
            }

            /* sample X, conditional on Y and S_old */
            j = nloc;
            rr[0] = xecomo[0][0][j];
            rr[1] = xecomo[1][0][j];
            xx[j] = back_sample3 (nupdate, nloc, j, 0, rr, ind, sx, pcf[j]);
            for (j = nloc - 1; j >= mmm; j--)
            {
               rr[0] = xecomo[0][0][j];
               rr[1] = xecomo[1][0][j];
               xx[j] =
                  back_sample3 (nupdate, nloc, j, xx[j + 1], rr, ind, sx,
                                pcf[j]);
            }

            /* assign new inheritance indicators to xx0 or xx1 and re-set genes */
            for (j = mmm; j <= nloc; j++)
            {
               jj = chromo[0][j];
               change_meiosis (0, nupdate, jj, ind, sx, xx[j]);
               set_FGL (jj);
               prob[c - 1][jj] =
                  seg_pen2 (c - 1, loci[jj].freq, jj, ngenes, &err);
            }
         }

         free_1d ((void *) &ind);
         free_1d ((void *) &sx);
      }
   }

   free_2d ((void *) &pp2, nloc + 1);
   free_2d ((void *) &pcf, nloc + 1);
   free_1d ((void *) &xx);

   return;
}

/* update meioses by 3 generations in component c */
/* for each component, consider all the loci jointly */

void update_3gen (int c, int *idlist, int nid, int *swlist, int nsw,
                  int *mrlist, int nmr, int nloc, int mm, double **prob,
                  int ngenes)
{
   int   t1, t2, i1, i2, ik, l, j, jj, j1, j2, t, mmm, err;
   int  *sx, *ind;
   int   nupdate1, nupdate2;

   double **pp2, **pcf;
   double rr[2];
   int  *xx;

   pp2 = alloc_dbl2d (nloc + 1, 2);
   pcf = alloc_dbl2d (nloc + 1, 2);
   al_int_vector (&xx, nloc + 1);

   mmm = mm;
   if (chromo[0][0] != 0)
      mmm = 0;

   /*
      for (t = 0; t < 2; t++)
      pcf[0][t] = 0;
    */

   permute (mrlist, nmr);               /* permute marriages for M-sampler */

   for (t1 = 0; t1 < nmr; t1++)
   {
      i1 = mrlist[t1];                  /* index of marriage in marriage list */
      nupdate1 = nupdate2 = marr[i1].nkidmeio; /* no. of meioses to the second generation 
                                                  in the i1 th marriage */

      /* first find nupdate2 = nupdate1 + no. of meioses to the third generation */

      for (j1 = 0; j1 < marr[i1].nkid; j1++) /* go through kids list in the i1 th marriage */
      {
         ik = marr[i1].kid[j1];         /* index of the j1 th kid in i1 th marriage */
         for (t2 = 0; t2 < indmarr[ik].nmarr; t2++) /* go through marriage list of ik */
         {
            i2 = indmarr[ik].marr[t2];  /* index of marriage of the j1 th kid in i1 th marriage */
            nupdate2 += marr[i2].nkidmeio;
         }
      }

      /* consider to update meioses to the second and third generations */
      if (nupdate2 > 0)
      {
         al_int_vector (&ind, nupdate2);
         al_int_vector (&sx, nupdate2);

         for (t = 0; t < nupdate1; t++)
         {
            l = marr[i1].kidmeio[t];
            ind[t] = (meiosis[l].ind).index;
            sx[t] = meiosis[l].sex;
         }
         for (t = nupdate1; t < nupdate2;)
         {
            for (j1 = 0; j1 < marr[i1].nkid; j1++) /* go through kids list in the i1 th marriage */
            {
               ik = marr[i1].kid[j1];   /* index of the j1 th kid in i1 th marriage */
               for (t2 = 0; t2 < indmarr[ik].nmarr; t2++) /* go through marriage list of ik */
               {
                  i2 = indmarr[ik].marr[t2]; /* index of marriage of the j1 th kid in i1 th marriage */
                  for (j2 = 0; j2 < marr[i2].nkidmeio; j2++)
                  {
                     l = marr[i2].kidmeio[j2];
                     ind[t] = (meiosis[l].ind).index;
                     sx[t] = meiosis[l].sex;
                     t++;
                  }
               }
            }
         }

         if (nupdate2 <= Max_update)    /* *** full 3-gen update *** */
         {
            do_msmpl2 (c, nloc, mm, ind, sx, nupdate2, prob, ngenes);
         }
         else                           /* *** restricted 3-gen update *** */
         {
            /* store all the penetrance values in pp2[NLOC][2] */

            for (j = mmm; j <= nloc; j++)
            {
               pp2[j][0] = prob[c - 1][j];

               change_meiosis (nupdate1, nupdate2, j, ind, sx, 10);
               set_FGL (j);
               pp2[j][1] = seg_pen2 (c - 1, loci[j].freq, j, ngenes, &err);
               prob[c - 1][j] = pp2[j][1];
            }

            /* compute P(X_j | Y_1, ..., Y_j, S_old) by "peeling" */
            if (mmm == 0)
            {
               j = 0;
               jj = chromo[0][j];
               fwd_compute4 (nupdate1, nupdate2, j, mm, NULL, ind, sx, pp2[jj],
                             NULL, pcf[j]);
            }
            for (j = 1; j <= nloc; j++)
            {
               jj = chromo[0][j];
               rr[0] = xecomo[0][0][j - 1];
               rr[1] = xecomo[1][0][j - 1];
               fwd_compute4 (nupdate1, nupdate2, j, mm, rr, ind, sx, pp2[jj],
                             pcf[j - 1], pcf[j]);
            }

            /* sample X, conditional on Y and S_old */

            j = nloc;
            rr[0] = xecomo[0][0][j];
            rr[1] = xecomo[1][0][j];
            xx[j] = back_sample4 (nupdate1, nupdate2, nloc, j, 0,
                                  rr, ind, sx, pcf[j]);
            for (j = nloc - 1; j >= mmm; j--)
            {
               rr[0] = xecomo[0][0][j];
               rr[1] = xecomo[1][0][j];
               xx[j] = back_sample4 (nupdate1, nupdate2, nloc, j, xx[j + 1],
                                     rr, ind, sx, pcf[j]);
            }

            /* assign new inheritance indicators to xx0 or xx1 and re-set genes */

            for (j = mmm; j <= nloc; j++)
            {
               if (xx[j] == 0)
               {
                  jj = chromo[0][j];
                  change_meiosis (nupdate1, nupdate2, jj, ind, sx, 10);
                  set_FGL (jj);
                  prob[c - 1][jj] =
                     seg_pen2 (c - 1, loci[jj].freq, jj, ngenes, &err);
               }
            }
         }

         free_1d ((void *) &ind);
         free_1d ((void *) &sx);
      }
   }

   free_2d ((void *) &pp2, nloc + 1);
   free_2d ((void *) &pcf, nloc + 1);
   free_1d ((void *) &xx);

   return;
}

/* change meiosis indicators xx0 and xx1 for updating type k
   k =  0: no change
   k =  1: flip kid's maternal meioses only
   k =  2: flip both maternal and paternal meioses
   k =  3: flip kid's paternal meioses only
   k = 10: swapping parent's maternal and paternal meioses
           and flipping all the kid's meioses
*/
/* nsize1: number of meioses involved in updating type 10
   nsize2: number of meioses involved in updating type 0,1,2,3
   j     : locus to perform updating type k 
   ind   : indices of individuals involved for each meiosis
   sx    : sex of each meiosis
*/

void change_meiosis (int nsize1, int nsize2, int j, int *ind, int *sx, int k)
{
   int   i, itemp;

   if (k == 0)
      return;

   if (k == 1 || k == 2)
   {
      for (i = 0; i < nsize2; i++)
         if (sx[i] == 0)
            xx0[j][ind[i]] = 1 - xx0[j][ind[i]];
   }

   if (k == 3 || k == 2)
   {
      for (i = 0; i < nsize2; i++)
         if (sx[i] == 1)
            xx1[j][ind[i]] = 1 - xx1[j][ind[i]];
   }

   if (k == 10)
   {
      for (i = 0; i < nsize1; i++)
      {
         if (sx[i] == 0)
         {
            itemp = xx0[j][ind[i]];
            xx0[j][ind[i]] = xx1[j][ind[i]];
            xx1[j][ind[i]] = itemp;
         }
      }
      for (i = nsize1; i < nsize2; i++)
      {
         if (sx[i] == 0)
            xx0[j][ind[i]] = 1 - xx0[j][ind[i]];
         if (sx[i] == 1)
            xx1[j][ind[i]] = 1 - xx1[j][ind[i]];
      }
   }

   return;
}
