/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#include "lsample.h"
#include "sample_order.h"

static int cmp_position (const void *s1, const void *s2)
{
   double x1, x2;

   x1 = (*((struct Locus **) s1))->fempos;
   x2 = (*((struct Locus **) s2))->fempos;
   if (x1 < x2)
      return -1;
   if (x1 > x2)
      return 1;
   return 0;
}

/* Sample order and segs for all loci in linkage groups of >1
 * conditional on current unordered genotype configuration */
int sample_order (struct Locus *locus, const int n_loci)
{
   int   i, nl, max_chrom, chrom;
   struct Locus **t_ord;

   for (i = max_chrom = 0; i < n_loci; i++)
      if (locus[i].chromosome > max_chrom)
         max_chrom = locus[i].chromosome;
   if (!(t_ord = malloc (sizeof (struct Locus *) * n_loci)))
      exit (EXIT_FAILURE);
   for (chrom = 1; chrom <= max_chrom; chrom++)
   {
      /* Collect all loci on chrom */
      for (nl = i = 0; i < n_loci; i++)
         if (locus[i].chromosome == chrom)
            t_ord[nl++] = locus + i;
      /* Only proceed if there are >1 loci on the chromosome */
      if (nl > 1)
      {
         qsort (t_ord, nl, sizeof (struct Locus *), cmp_position);
         for (i = 0; i < nl; i++)
         {
            /* Find which orders and segs are fixed */
            check_fixed (t_ord[i]);
         }
      }
   }
   free (t_ord);
   return 0;
}
