/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/*****************************************************************************
  routine name: sampling2.c

  Take out the MCMC sampling portion of lm_auto program, 
  to separate from scoring of results, and accommodate burn-in etc.
                                                                   EAT Sep 2002

  set up permutation arrays for loci and switches moved from main  YJS Jan 2005

  allow for individual, sibs and generations update in M-sampler   LPT Feb 2005
 
  Macro constants PMEIO etc and MAXNMupdate changed to globals     EAT Apr 2007

******************************************************************************/

#include "lsample.h"
#include "msample.h"
#include "mulmsample.h"

static int **perm_meio = NULL;          /* permutation matrix of meioses 
                                           for each component */
static int **perm_ind = NULL;           /* permutation matrix of individuals 
                                           for each component */
static int **perm_marr = NULL;          /* permutation martix of marriages
                                           for each component */
static int *perm_loc = NULL;            /* permutation array of loci */

/* allocate memory and assign initial values to perm_meio,
   perm_ind and perm_loc, which are used in sampling_mul () 
*/

void setup_sampling2 (int nloc)
{
   int   nsw, nms, nid, nmr, i, c, k;
   Indlist *pcurr;

   nsw = 0;
   for (c = 1; c <= n_compnts; c++)
      nsw += cmplist[c].nmeioses;

   if (nsw == 0)
      return;

   perm_meio = malloc (n_compnts * sizeof (int *));
   alloc_chk (WHERE, perm_meio);
   for (c = 1; c <= n_compnts; c++)
   {
      nms = cmplist[c].nmeioses;
      perm_meio[c - 1] = malloc (nms * sizeof (int));
      alloc_chk (WHERE, perm_meio[c - 1]);
      for (k = 0; k < nms; k++)
         perm_meio[c - 1][k] = cmplist[c].meioses[k];
   }

   if (Scan)
   {
      perm_ind = malloc (n_compnts * sizeof (int *));
      alloc_chk (WHERE, perm_ind);
      perm_marr = malloc (n_compnts * sizeof (int *));
      alloc_chk (WHERE, perm_marr);

      for (c = 1; c <= n_compnts; c++)
      {
         nid = cmplist[c].size;
         perm_ind[c - 1] = malloc (nid * sizeof (int));
         alloc_chk (WHERE, perm_ind[c - 1]);
         pcurr = cmplist[c].people->next;
         k = 0;
         while (pcurr && (k < nid))
         {
            perm_ind[c - 1][k] = pcurr->current->index;
            pcurr = pcurr->next;
            k++;
         }

         nmr = cmpmarr[c - 1].nmarr;
         perm_marr[c - 1] = malloc (nmr * sizeof (int));
         alloc_chk (WHERE, perm_marr[c - 1]);
         for (k = 0; k < nmr; k++)
            perm_marr[c - 1][k] = cmpmarr[c - 1].marr[k];
      }

      al_int_vector (&perm_loc, nloc + 1);
      for (i = 0; i <= nloc; ++i)
         perm_loc[i] = i;
   }
   return;
}

void free_sampling2 (void)
{
   free_2d ((void *) &perm_meio, n_compnts);
   free_2d ((void *) &perm_ind, n_compnts);
   free_2d ((void *) &perm_marr, n_compnts);
   free_1d ((void *) &perm_loc);
   return;
}

/* t = 1, 2, ... indices of mcmc iterations */

void sampling2 (int t, int mm, int nloc, int ngenes, double **prob)
/* mm=0 if L-sample and M-sample sample all loci, mm=1 if markers only  */
{
   int   k, l, i, jj, c, nms, nid, nmr;
   double ptemp;
   int   nupdate;
   int  *sx, *ind, *ptms, *ptid, *ptmr;

   if (rans () < LSamplProb)
   {                                    /* ***** Do L-sample *****  */
      /*
         printf("\n === L-sampling, t=%d", t); getchar();
       */

      if (!((t) % ChkIt))
      {
         printf (" Starting L-sampler iteration %d\n", t);
         fflush (stdout);
      }
      if (Scan)                         /* ****   scan   **** */
      {
         permute (perm_loc, nloc + 1);  /* permute loci for L-sampler */
         for (k = 0; k <= nloc; ++k)
         {                              /* do l-sampling for each locus */
            jj = perm_loc[k];
            if (jj >= mm)               /* if mm=1, don't do jj=0  */
               (void) do_lsmpl (nloc, jj, prob, ngenes);
         }
      }
      else
      {                                 /* ****   step   **** */

         jj = intran (mm, nloc);        /* choose a random locus
                                         * do l-sampling for this locus */
         (void) do_lsmpl (nloc, jj, prob, ngenes);

      }
   }
   else
   {                                    /* ***** Do M-sample *****  */
      if (!((t) % ChkIt))
      {
         printf (" Starting M-sampler iteration %d\n", t);
         fflush (stdout);
      }
      if (Scan)                         /* ****   scan   **** */
      {
         for (c = 1; c <= n_compnts; c++)
         {
            ptemp = rans ();

            ptms = perm_meio[c - 1];
            nms = cmplist[c].nmeioses;
            ptid = perm_ind[c - 1];
            nid = cmplist[c].size;
            ptmr = perm_marr[c - 1];
            nmr = cmpmarr[c - 1].nmarr;

            if (ptemp < P_meio)
               update_meio (c, ptid, nid, ptms, nms, ptmr, nmr, nloc, mm, prob,
                            ngenes);
            else if (ptemp < P_meio + P_ind)
               update_ind (c, ptid, nid, ptms, nms, ptmr, nmr, nloc, mm, prob,
                           ngenes);
            else if (ptemp < P_meio + P_ind + P_sib)
               update_sib (c, ptid, nid, ptms, nms, ptmr, nmr, nloc, mm, prob,
                           ngenes);
            else
               update_3gen (c, ptid, nid, ptms, nms, ptmr, nmr, nloc, mm, prob,
                            ngenes);
         }
      }
      else                              /* ****   step   **** */
      {
         c = intran (1, n_compnts);     /* choose a component */
         ptms = perm_meio[c - 1];
         nms = cmplist[c].nmeioses;     /* nms is the number of meioses in c */
         if (nms > 0)
         {
            permute (ptms, nms);

            /* number of meioses to update */
            i = intran (1, Max_update);
            nupdate = min (i, nms);

            al_int_vector (&(ind), nupdate);
            al_int_vector (&(sx), nupdate);
            for (i = 0; i < nupdate; i++)
            {
               l = ptms[i];
               ind[i] = (meiosis[l].ind).index;
               sx[i] = meiosis[l].sex;
            }
            do_msmpl2 (c, nloc, mm, ind, sx, nupdate, prob, ngenes);
            free_1d ((void *) &ind);
            free_1d ((void *) &sx);
         }
      }
   }
   return;
}
